@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pluto_constants.h,v 1.33.2.3 2005-11-16 22:41:30 ken Exp $
+ * RCSID $Id: pluto_constants.h,v 1.39 2005/10/03 19:58:12 mcr Exp $
  */
 
 /* Control and lock pathnames */
@@ -29,13 +29,11 @@ enum kernel_interface {
   NO_KERNEL = 1,
   AUTO_PICK = 2,
   USE_KLIPS = 3,
-  USE_NETKEY= 4
+  USE_NETKEY= 4,
+  USE_WIN2K = 5,
+  USE_FUNK  = 6,
+  USE_MASTKLIPS = 7,
 };
-extern enum kernel_interface kern_interface;
-extern enum_names kern_interface_names;
-
-
-extern enum_names dpd_action_names;
 
 /* RFC 3706 Dead Peer Detection */
 enum dpd_action {
@@ -47,8 +45,6 @@ enum dpd_action {
 
 /* Timer events */
 
-extern enum_names timer_event_names;
-
 enum event_type {
     EVENT_NULL,	/* non-event */
     EVENT_REINIT_SECRET,	/* Refresh cookie secret */
@@ -70,6 +66,7 @@ enum event_type {
 #define EVENT_REINIT_SECRET_DELAY		3600 /* 1 hour */
 #define EVENT_CRYPTO_FAILED_DELAY               300
 #define EVENT_RETRANSMIT_DELAY_0		10   /* 10 seconds */
+#define EVENT_GIVEUP_ON_DNS_DELAY               300  /* 5 minutes for DNS */
 
 /*
  * cryptographic helper operations.
@@ -85,8 +82,6 @@ enum pluto_crypto_requests {
   pcr_compute_dh     = 8,  /* perform phase 2 PFS DH */
 };
 
-extern enum_names pluto_cryptoop_names;
-
 /*
  * operational importance of this cryptographic operation.
  * this determines if the operation will be dropped (because the other
@@ -95,11 +90,12 @@ extern enum_names pluto_cryptoop_names;
  * we should do it all costs.
  */
 enum crypto_importance {
-  pcim_stranger_crypto = 1,
-  pcim_known_crypto    = 2,
-  pcim_ongoing_crypto  = 3,
-  pcim_local_crypto    = 4,
-  pcim_demand_crypto   = 5
+	pcim_notset_crypto=0,
+	pcim_stranger_crypto = 1,
+	pcim_known_crypto    = 2,
+	pcim_ongoing_crypto  = 3,
+	pcim_local_crypto    = 4,
+	pcim_demand_crypto   = 5
 };
 
 /* status for state-transition-function
@@ -108,7 +104,7 @@ enum crypto_importance {
 
 typedef enum {
     STF_IGNORE,	/* don't respond */
-    STF_INLINE,        /* set to this on second time through complete_state_trans */
+    STF_INLINE,	/* set to this on second time through complete_state_trans */
     STF_SUSPEND,    /* unfinished -- don't release resources */
     STF_OK,	/* success */
     STF_INTERNAL_ERROR,	/* discard everything, we failed */
@@ -116,11 +112,11 @@ typedef enum {
 			 * so just ignore the message, and let them retransmit.
 			 */
     STF_FATAL,          /* just stop. we can't continue. */
+    STF_STOLEN,         /* only used by TaProoM */
     STF_FAIL,	        /* discard everything, something failed.  notification_t added.
 			 * values STF_FAIL + x are notifications.
 			 */
 } stf_status;
-extern enum_names stfstatus_name;
 
 /* Misc. stuff */
 
@@ -143,7 +139,6 @@ extern enum_names stfstatus_name;
  * in whack.c.  A change to WHACK_MAGIC in whack.h will be required too.
  */
 #if !defined(NO_DEBUG)
-extern const char *const debug_bit_names[];
 
 #define DBG_RAW		LELEM(0)	/* raw packet I/O */
 #define DBG_CRYPT	LELEM(1)	/* encryption/decryption of messages */
@@ -152,6 +147,7 @@ extern const char *const debug_bit_names[];
 #define DBG_CONTROL	LELEM(4)	/* control flow within Pluto */
 #define DBG_LIFECYCLE	LELEM(5)	/* SA lifecycle */
 #define DBG_KLIPS	LELEM(6)	/* messages to KLIPS */
+#define DBG_NETKEY	LELEM(6)	/* messages to NETKEY/XFRM */
 #define DBG_DNS		LELEM(7)	/* DNS activity */
 #define DBG_OPPO	LELEM(8)	/* opportunism */
 #define DBG_CONTROLMORE LELEM(9)	/* more detailed debugging */
@@ -160,6 +156,7 @@ extern const char *const debug_bit_names[];
 #define DBG_NATT        LELEM(11)       /* debugging of NAT-traversal */
 #define DBG_X509        LELEM(12)       /* X.509/pkix verify, cert retrival */
 #define DBG_DPD         LELEM(13)       /* DPD items */
+#define DBG_OPPOINFO    LELEM(14)       /* log various informational things about oppo/%trap-keying */
 #define DBG_PRIVATE	LELEM(20)	/* private information: DANGER! */
 
 #define IMPAIR0	21	/* first bit for IMPAIR_* */
@@ -204,9 +201,6 @@ extern const char *const debug_bit_names[];
  * other important details.
  */
 
-extern enum_names state_names;
-extern const char *const state_story[];
-
 enum state_kind {
     STATE_UNDEFINED=0,	/* 0 -- most likely accident */
 
@@ -256,8 +250,6 @@ enum state_kind {
     STATE_IKE_ROOF
 
 };
-extern enum_names state_names;
-extern enum_names state_stories;
 
 enum phase1_role {
   INITIATOR=1,
@@ -302,8 +294,6 @@ enum phase1_role {
  * Ordered (mostly) by concreteness.  Order is exploited.
  */
 
-extern enum_names connection_kind_names;
-
 enum connection_kind {
     CK_GROUP,		/* policy group: instantiates to template */
     CK_TEMPLATE,	/* abstract connection, with wildcard */
@@ -319,8 +309,6 @@ enum connection_kind {
  * or HAS_IPSEC_POLICY.
  */
 
-extern enum_names routing_story;
-
 /* note that this is assumed to be ordered! */
 enum routing_t {
     RT_UNROUTED,	/* unrouted */
@@ -337,8 +325,6 @@ enum routing_t {
 #define erouted(rs) ((rs) != RT_UNROUTED)
 #define shunt_erouted(rs) (erouted(rs) && (rs) != RT_ROUTED_TUNNEL)
 
-extern enum_names certpolicy_type_names;
-
 enum certpolicy {
   cert_neversend   = 1,
   cert_sendifasked = 2,    /* the default */
@@ -347,7 +333,7 @@ enum certpolicy {
 };
 
 /* this is the default setting. */
-#define cert_defaultcertpolicy cert_alwayssend
+#define cert_defaultcertpolicy cert_sendifasked
 
 
 /* Policies for establishing an SA
@@ -358,64 +344,70 @@ enum certpolicy {
  * in sync!
  */
 
-extern const char *const sa_policy_bit_names[];
 extern const char *prettypolicy(lset_t policy);
 
 /* ISAKMP auth techniques (none means never negotiate) */
-#define POLICY_PSK           LELEM(0)
-#define POLICY_RSASIG        LELEM(1)
-
+enum pluto_policy {
+	POLICY_PSK     = LELEM(0), 
+	POLICY_RSASIG  = LELEM(1),
 #define POLICY_ISAKMP_SHIFT	0	/* log2(POLICY_PSK) */
 
 /* policies that affect ID types that are acceptable - RSA, PSK, XAUTH */
-#define POLICY_ID_AUTH_MASK	LRANGES(POLICY_PSK, POLICY_RSASIG)
+        POLICY_ID_AUTH_MASK=LRANGES(POLICY_PSK, POLICY_RSASIG), 
 
 /* policies that affect choices of proposal, note, does not include XAUTH */
 #define POLICY_ISAKMP(x,xs,xc)	(((x) & LRANGES(POLICY_PSK, POLICY_RSASIG)) + \
                                  ((xs)*4) + ((xc)*8))
 
 /* Quick Mode (IPSEC) attributes */
-#define POLICY_ENCRYPT       LELEM(2)	/* must be first of IPSEC policies */
-#define POLICY_AUTHENTICATE  LELEM(3)	/* must be second */
-#define POLICY_COMPRESS      LELEM(4)	/* must be third */
-#define POLICY_TUNNEL        LELEM(5)
-#define POLICY_PFS           LELEM(6)
-#define POLICY_DISABLEARRIVALCHECK  LELEM(7)	/* supress tunnel egress address checking */
+	POLICY_ENCRYPT = LELEM(2),	/* must be first of IPSEC policies */
+	POLICY_AUTHENTICATE=LELEM(3),	/* must be second */
+	POLICY_COMPRESS=LELEM(4),	/* must be third */
+	POLICY_TUNNEL  = LELEM(5),
+	POLICY_PFS     = LELEM(6),
+	POLICY_DISABLEARRIVALCHECK = LELEM(7),	/* supress tunnel egress address checking */
 
 #define POLICY_IPSEC_SHIFT	2	/* log2(POLICY_ENCRYPT) */
-#define POLICY_IPSEC_MASK	LRANGES(POLICY_ENCRYPT, POLICY_DISABLEARRIVALCHECK)
+	POLICY_IPSEC_MASK = LRANGES(POLICY_ENCRYPT, POLICY_DISABLEARRIVALCHECK),
 
 /* shunt attributes: what to do when routed without tunnel (2 bits) */
-#define POLICY_SHUNT_SHIFT	8	/* log2(POLICY_SHUNT_PASS) */
-#define POLICY_SHUNT_MASK	(03ul << POLICY_SHUNT_SHIFT)
-
-#define POLICY_SHUNT_TRAP	(0ul << POLICY_SHUNT_SHIFT) /* default: negotiate */
-#define POLICY_SHUNT_PASS	(1ul << POLICY_SHUNT_SHIFT)
-#define POLICY_SHUNT_DROP	(2ul << POLICY_SHUNT_SHIFT)
-#define POLICY_SHUNT_REJECT	(3ul << POLICY_SHUNT_SHIFT)
+	POLICY_SHUNT_SHIFT = 8,	/* log2(POLICY_SHUNT_PASS) */
+	POLICY_SHUNT_MASK  = (03ul << POLICY_SHUNT_SHIFT),
+	POLICY_SHUNT_TRAP  = (0ul << POLICY_SHUNT_SHIFT), /* default: negotiate */
+	POLICY_SHUNT_PASS  = (1ul << POLICY_SHUNT_SHIFT), 
+	POLICY_SHUNT_DROP  = (2ul << POLICY_SHUNT_SHIFT),
+	POLICY_SHUNT_REJECT=(3ul << POLICY_SHUNT_SHIFT),
 
 /* fail attributes: what to do with failed negotiation (2 bits) */
 
-#define POLICY_FAIL_SHIFT	10	/* log2(POLICY_FAIL_PASS) */
-#define POLICY_FAIL_MASK	(03ul << POLICY_FAIL_SHIFT)
+	POLICY_FAIL_SHIFT  = 10,	/* log2(POLICY_FAIL_PASS) */
+	POLICY_FAIL_MASK   = (03ul << POLICY_FAIL_SHIFT),
 
-#define POLICY_FAIL_NONE     (0ul << POLICY_FAIL_SHIFT) /* default */
-#define POLICY_FAIL_PASS     (1ul << POLICY_FAIL_SHIFT)
-#define POLICY_FAIL_DROP     (2ul << POLICY_FAIL_SHIFT)
-#define POLICY_FAIL_REJECT   (3ul << POLICY_FAIL_SHIFT)
+	POLICY_FAIL_NONE   = (0ul << POLICY_FAIL_SHIFT), /* default */
+	POLICY_FAIL_PASS   = (1ul << POLICY_FAIL_SHIFT),
+	POLICY_FAIL_DROP   = (2ul << POLICY_FAIL_SHIFT),
+	POLICY_FAIL_REJECT = (3ul << POLICY_FAIL_SHIFT),
 
 /* connection policy
  * Other policies could vary per state object.  These live in connection.
  */
-#define POLICY_DONT_REKEY   LELEM(12)   /* don't rekey state either Phase */
-#define POLICY_OPPO         LELEM(13)   /* is this opportunistic? */
-#define POLICY_GROUP        LELEM(14)   /* is this a group template? */
-#define POLICY_GROUTED      LELEM(15)   /* do we want this group routed? */
-#define POLICY_UP           LELEM(16)   /* do we want this up? */
-#define POLICY_XAUTH        LELEM(17)   /* do we offer XAUTH? */
-#define POLICY_MODECFG_PULL LELEM(18)   /* is modecfg pulled by client? */
-#define POLICY_AGGRESSIVE   LELEM(19)   /* do we do aggressive mode? */
-
+	POLICY_DONT_REKEY   = LELEM(12),   /* don't rekey state either Phase */
+	POLICY_OPPO         = LELEM(13),   /* is this opportunistic? */
+	POLICY_GROUP        = LELEM(14),   /* is this a group template? */
+	POLICY_GROUTED      = LELEM(15),   /* do we want this group routed? */
+	POLICY_UP           = LELEM(16),   /* do we want this up? */
+	POLICY_XAUTH        = LELEM(17),   /* do we offer XAUTH? */
+	POLICY_MODECFG_PULL = LELEM(18),   /* is modecfg pulled by client? */
+	POLICY_AGGRESSIVE   = LELEM(19),   /* do we do aggressive mode? */
+	POLICY_PERHOST      = LELEM(20),   /* should we specialize the policy to the host? */
+	POLICY_SUBHOST      = LELEM(21),   /* if the policy applies below the host level (TCP/UDP/SCTP ports), */
+	POLICY_PERPROTO     = LELEM(22),   /* should we specialize the policy to the protocol? */
+	POLICY_OVERLAPIP    = LELEM(23),   /* can two conns that have subnet=vhost: declare the same IP? */
+	POLICY_MODECFGDNS1  = LELEM(24),   /* should we offer a DNS server IP */
+	POLICY_MODECFGDNS2  = LELEM(25),   /* should we offer another DNS server IP */
+	POLICY_MODECFGWINS1 = LELEM(26),   /* should we offer a WINS server IP */
+	POLICY_MODECFGWINS2 = LELEM(27),   /* should we offer another WINS server IP */
+};
 
 /* Any IPsec policy?  If not, a connection description
  * is only for ISAKMP SA, not IPSEC SA.  (A pun, I admit.)
@@ -432,9 +424,6 @@ extern const char *prettypolicy(lset_t policy);
  * draft-ietf-ipsec-ike-01.txt appendix A
  */
 
-extern enum_names oakley_attr_names;
-extern const char *const oakley_attr_bit_names[];
-
 #define OAKLEY_ENCRYPTION_ALGORITHM    1
 #define OAKLEY_HASH_ALGORITHM          2
 #define OAKLEY_AUTHENTICATION_METHOD   3
@@ -453,15 +442,10 @@ extern const char *const oakley_attr_bit_names[];
 #define OAKLEY_GROUP_ORDER            16	/* B/V */
 #define OAKLEY_BLOCK_SIZE             17
 
-/* for each Oakley attribute, which enum_names describes its values? */
-extern enum_names *oakley_attr_val_descs[];
-
 /* IPsec DOI attributes
  * RFC2407 The Internet IP security Domain of Interpretation for ISAKMP 4.5
  */
 
-extern enum_names ipsec_attr_names;
-
 #define SA_LIFE_TYPE             1
 #define SA_LIFE_DURATION         2	/* B/V */
 #define GROUP_DESCRIPTION        3
@@ -473,7 +457,6 @@ extern enum_names ipsec_attr_names;
 #define COMPRESS_PRIVATE_ALG     9	/* B/V */
 
 /* for each IPsec attribute, which enum_names describes its values? */
-extern enum_names *ipsec_attr_val_descs[];
 
 /* SA Lifetime Type attribute
  * RFC2407 The Internet IP security Domain of Interpretation for ISAKMP 4.5
@@ -489,8 +472,6 @@ extern enum_names *ipsec_attr_val_descs[];
  * The value of SA_LIFE_DURATION_MAXIMUM is our local policy.
  */
 
-extern enum_names sa_lifetime_names;
-
 #define SA_LIFE_TYPE_SECONDS   1
 #define SA_LIFE_TYPE_KBYTES    2
 
@@ -506,8 +487,6 @@ extern enum_names sa_lifetime_names;
 
 /* Encapsulation Mode attribute */
 
-extern enum_names enc_mode_names;
-
 #define ENCAPSULATION_MODE_UNSPECIFIED 0	/* not legal -- used internally */
 #define ENCAPSULATION_MODE_TUNNEL      1
 #define ENCAPSULATION_MODE_TRANSPORT   2
@@ -526,7 +505,6 @@ extern enum_names enc_mode_names;
 
 /* Auth Algorithm attribute */
 
-extern enum_names auth_alg_names, extended_auth_alg_names;
 
 #define AUTH_ALGORITHM_NONE        0	/* our private designation */
 #define AUTH_ALGORITHM_HMAC_MD5    1
@@ -546,7 +524,6 @@ extern enum_names auth_alg_names, extended_auth_alg_names;
  * For no particular reason, we chose one hour.
  * The value of OAKLEY_ISAKMP_SA_LIFETIME_MAXIMUM is our local policy.
  */
-extern enum_names oakley_lifetime_names;
 
 #define OAKLEY_LIFE_SECONDS   1
 #define OAKLEY_LIFE_KILOBYTES 2
@@ -559,6 +536,7 @@ enum pubkey_source
     PUBKEY_NOTSET       = 0,
     PUBKEY_DNS          = 1,
     PUBKEY_DNSONDEMAND  = 2,
+    PUBKEY_CERTIFICATE  = 3,
     PUBKEY_PREEXCHANGED = LOOSE_ENUM_OTHER,
 };
 
@@ -571,43 +549,12 @@ enum keyword_host {
     KH_OPPO         = 4,
     KH_OPPOGROUP    = 5,
     KH_GROUP        = 6,
+    KH_IPHOSTNAME   = 7,                /* host_addr invalid, only string */
     KH_IPADDR       = LOOSE_ENUM_OTHER,
 };
 
-
-/* socket address family info */
-
-struct af_info
-{
-    int af;
-    const char *name;
-    size_t ia_sz;
-    size_t sa_sz;
-    int mask_cnt;
-    u_int8_t id_addr, id_subnet, id_range;
-    const ip_address *any;
-    const ip_subnet *none;	/* 0.0.0.0/32 or IPv6 equivalent */
-    const ip_subnet *all;	/* 0.0.0.0/0 or IPv6 equivalent */
-};
-
-extern const struct af_info
-    af_inet4_info,
-    af_inet6_info;
-
-extern const struct af_info *aftoinfo(int af);
-
-extern enum_names af_names;
-
-#define subnetisaddr(sn, a) (subnetishost(sn) && addrinsubnet((a), (sn)))
-extern bool subnetisnone(const ip_subnet *sn);
-
 /* BIND enumerated types */
 
-extern enum_names
-    rr_qtype_names,
-    rr_type_names,
-    rr_class_names;
-
 /* How authenticated is info that might have come from DNS?
  * In order of increasing confidence.
  */
@@ -635,12 +582,9 @@ enum PrivateKeyKind {
     PPK_PSK = 1,
     /* PPK_DSS, */	/* not implemented */
     PPK_RSA = 3,
-    PPK_PIN = 4
+    PPK_PIN = 4,
+    PPK_XAUTH=5,
 };
-extern enum_names ppk_names;
-
-/* natt traversal types */
-extern const char *const natt_type_bitnames[];
 
 
 
