@@ -28,7 +28,7 @@
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
 #include <openswan/passert.h>
-#include <pfkeyv2.h>
+#include <openswan/pfkeyv2.h>
 
 #include "constants.h"
 #include "alg_info.h"
@@ -88,7 +88,7 @@ alg_info_esp_sadb2aa(int sadb_aalg)
  * 	Search enum_name array with in prefixed uppercase
  */
 int
-alg_enum_search_prefix (enum_names *ed, const char *prefix, const char *str, int strlen)
+alg_enum_search_prefix (enum_names *ed, const char *prefix, const char *str, int str_len)
 {
 	char buf[64];
 	char *ptr;
@@ -97,7 +97,7 @@ alg_enum_search_prefix (enum_names *ed, const char *prefix, const char *str, int
 
 	for (ptr=buf; *prefix; *ptr++=*prefix++, len--);
 
-	while (strlen--&&len--&&*str) *ptr++=toupper(*str++);
+	while (str_len--&&len--&&*str) *ptr++=toupper(*str++);
 	*ptr=0;
 
 	DBG(DBG_CRYPT, DBG_log("enum_search_prefix () "
@@ -112,14 +112,14 @@ alg_enum_search_prefix (enum_names *ed, const char *prefix, const char *str, int
 int
 alg_enum_search_ppfix (enum_names *ed, const char *prefix
 		   , const char *postfix, const char *str
-		   , int strlen)
+		   , int str_len)
 {
 	char buf[64];
 	char *ptr;
 	int ret;
 	int len=sizeof(buf)-1;	/* reserve space for final \0 */
 	for (ptr=buf; *prefix; *ptr++=*prefix++, len--);
-	while (strlen--&&len--&&*str) *ptr++=toupper(*str++);
+	while (str_len--&&len--&&*str) *ptr++=toupper(*str++);
 	while (len--&&*postfix) *ptr++=*postfix++;
 	*ptr=0;
 	DBG(DBG_CRYPT, DBG_log("enum_search_ppfixi () "
@@ -254,8 +254,37 @@ alg_info_esp_add (struct alg_info *alg_info,
 	}
 }
 
+/*	
+ *	Add AH alg info _with_ logic (policy):
+ */
+static void
+alg_info_ah_add (struct alg_info *alg_info,
+		  int ealg_id, int ek_bits,
+		  int aalg_id, int ak_bits,
+		  int modp_id, bool permit_manconn)
+{
+    if(aalg_id > 0 ||
+       (permit_manconn && aalg_id == 0))
+    {
+	__alg_info_esp_add((struct alg_info_esp *)alg_info,
+			   ealg_id, ek_bits,
+			   aalg_id, ak_bits);
+    }
+    else
+    {
+	/*	Policy: default to MD5 and SHA1 */
+	__alg_info_esp_add((struct alg_info_esp *)alg_info,
+			   ealg_id, ek_bits,				\
+			   AUTH_ALGORITHM_HMAC_MD5, ak_bits);
+	__alg_info_esp_add((struct alg_info_esp *)alg_info,
+			   ealg_id, ek_bits,				\
+			   AUTH_ALGORITHM_HMAC_SHA1, ak_bits);
+    }
+}
+
 static const char *parser_state_esp_names[] = {
 	"ST_INI",
+	"ST_INI_AA",
 	"ST_EA",
 	"ST_EA_END",	
 	"ST_EK",
@@ -330,6 +359,18 @@ parser_machine(struct parser_context *p_ctx)
 	    }
 	    p_ctx->err="No alphanum. char initially found";
 	    goto err;
+
+	case ST_INI_AA:
+	    if (isspace(ch))
+		break;
+	    if (isalnum(ch)) {
+		*(p_ctx->aalg_str++)=ch;
+		parser_set_state(p_ctx, ST_AA);
+		break;
+	    }
+	    p_ctx->err="No alphanum. char initially found";
+	    goto err;
+
 	case ST_EA:
 	    if (isalpha(ch) || ch == '_') {
 		*(p_ctx->ealg_str++)=ch;
@@ -466,6 +507,8 @@ parser_init_esp(struct parser_context *p_ctx)
     p_ctx->ealg_str=p_ctx->ealg_buf;
     p_ctx->aalg_str=p_ctx->aalg_buf;
     p_ctx->modp_str=p_ctx->modp_buf;
+    p_ctx->ealg_permit = TRUE;
+    p_ctx->aalg_permit = TRUE;
     p_ctx->state=ST_INI;
     
     p_ctx->ealg_getbyname=ealg_getbyname_esp;
@@ -473,6 +516,28 @@ parser_init_esp(struct parser_context *p_ctx)
 
 }
 
+/*	
+ *	Must be called for each "new" char, with new
+ *	character in ctx.ch
+ */
+static void
+parser_init_ah(struct parser_context *p_ctx)
+{
+    memset(p_ctx, 0, sizeof (*p_ctx));
+
+    p_ctx->protoid=PROTO_IPSEC_AH;
+    p_ctx->ealg_str=NULL;
+    p_ctx->ealg_permit = FALSE;
+    p_ctx->aalg_str=p_ctx->aalg_buf;
+    p_ctx->aalg_permit = TRUE;
+    p_ctx->modp_str=p_ctx->modp_buf;
+    p_ctx->state=ST_INI_AA;
+    
+    p_ctx->ealg_getbyname=NULL;
+    p_ctx->aalg_getbyname=aalg_getbyname_esp;
+
+}
+
 static int
 parser_alg_info_add(struct parser_context *p_ctx
 		    , struct alg_info *alg_info
@@ -488,8 +553,8 @@ parser_alg_info_add(struct parser_context *p_ctx
 	int modp_id = 0;
 	const struct oakley_group_desc *gd;
 
-	ealg_id=aalg_id=0;
-	if (*p_ctx->ealg_buf) {
+	ealg_id=aalg_id=-1;
+	if (p_ctx->ealg_permit && *p_ctx->ealg_buf) {
 	    ealg_id=p_ctx->ealg_getbyname(p_ctx->ealg_buf, strlen(p_ctx->ealg_buf));
 	    if (ealg_id==ESP_MAGIC_ID) {
 		ealg_id=p_ctx->eklen;
@@ -504,7 +569,7 @@ parser_alg_info_add(struct parser_context *p_ctx
 				   p_ctx->ealg_buf,
 				   ealg_id));
 	}
-	if (*p_ctx->aalg_buf) {
+	if (p_ctx->aalg_permit && *p_ctx->aalg_buf) {
 	    aalg_id=p_ctx->aalg_getbyname(p_ctx->aalg_buf, strlen(p_ctx->aalg_buf));
 	    if (aalg_id<0) {
 		p_ctx->err="hash_alg not found";
@@ -568,10 +633,6 @@ alg_info_parse_str (struct alg_info *alg_info
 	/* use default if nul esp string */
 	if (!*alg_str) {
 	    (*alg_info_add)(alg_info, 0, 0, 0, 0, 0, 0);
-	} else {
-		/* This is necessary to have mandatory proposals
-		   add the flag as soon as an explicit proposal is set */
-		alg_info->alg_info_flags |= ALG_INFO_F_STRICT;
 	}
 
 	for(ret=0,ptr=alg_str;ret<ST_EOF;) {
@@ -629,6 +690,46 @@ alg_info_parse_str (struct alg_info *alg_info
 	return -1;
 }
 
+static bool
+alg_info_discover_pfsgroup_hack(struct alg_info_esp *aie
+				, char *esp_buf
+				, const char **err_p)
+{
+    char *pfs_name;
+    static char err_buf[256];
+    int ret;
+    
+    pfs_name=index(esp_buf, ';');
+    
+    if(pfs_name) {
+	*pfs_name='\0';
+	pfs_name++;
+
+	/* if pfs strings AND first char is not '0' */
+	if (*pfs_name && pfs_name[0]!='0') {
+	    ret=modp_getbyname_esp(pfs_name, strlen(pfs_name));
+	    if (ret<0) {
+		/* Bomb if pfsgroup not found */
+		DBG(DBG_CRYPT, DBG_log("alg_info_*_create_from_str(): "
+				       "pfsgroup \"%s\" not found",
+				       pfs_name));
+		if (*err_p) {
+		    snprintf(err_buf, sizeof(err_buf),
+			     "pfsgroup \"%s\" not found",
+			     pfs_name);
+		    *err_p=err_buf;
+		}
+		return FALSE;
+	    }
+	    aie->esp_pfsgroup=ret;
+	}
+    } else
+	aie->esp_pfsgroup = 0;
+
+    return TRUE;
+}
+
+
 struct alg_info_esp *
 alg_info_esp_create_from_str (const char *alg_str
 			      , const char **err_p
@@ -636,8 +737,6 @@ alg_info_esp_create_from_str (const char *alg_str
 {
     struct alg_info_esp *alg_info_esp;
     char esp_buf[256];
-    static char err_buf[256];
-    char *pfs_name;
     int ret =0;
 
     /*
@@ -649,38 +748,14 @@ alg_info_esp_create_from_str (const char *alg_str
 
     if (!alg_info_esp) goto out;
 
-    pfs_name=index (alg_str, ';');
+    strcpy(esp_buf, alg_str);
+    if(!alg_info_discover_pfsgroup_hack(alg_info_esp, esp_buf, err_p)) {
+	return NULL;
+    }
 
-    if (pfs_name) {
-	memcpy(esp_buf, alg_str, pfs_name-alg_str);
-	esp_buf[pfs_name-alg_str] = 0;
-	alg_str=esp_buf;
-	pfs_name++;
-
-	/* if pfs strings AND first char is not '0' */
-	if (*pfs_name && pfs_name[0]!='0') {
-	    ret=modp_getbyname_esp(pfs_name, strlen(pfs_name));
-	    if (ret<0) {
-		/* Bomb if pfsgroup not found */
-		DBG(DBG_CRYPT, DBG_log("alg_info_esp_create_from_str(): "
-				       "pfsgroup \"%s\" not found",
-				       pfs_name));
-		if (*err_p) {
-		    snprintf(err_buf, sizeof(err_buf),
-			     "pfsgroup \"%s\" not found",
-			     pfs_name);
-		    *err_p=err_buf;
-		}
-		goto out;
-	    }
-	    alg_info_esp->esp_pfsgroup=ret;
-	}
-    } else
-	alg_info_esp->esp_pfsgroup = 0;
-    
     alg_info_esp->alg_info_protoid=PROTO_IPSEC_ESP;
     ret=alg_info_parse_str((struct alg_info *)alg_info_esp
-			   , alg_str, err_p
+			   , esp_buf, err_p
 			   , parser_init_esp
 			   , alg_info_esp_add
 			   , NULL
@@ -696,6 +771,43 @@ alg_info_esp_create_from_str (const char *alg_str
     
 }
 
+struct alg_info_esp *
+alg_info_ah_create_from_str (const char *alg_str
+			     , const char **err_p
+			     , bool permitmann)
+{
+    struct alg_info_esp *alg_info_esp;
+    char esp_buf[256];
+    int ret =0;
+
+    /*
+     * 	alg_info storage should be sized dynamically
+     * 	but this may require 2passes to know
+     * 	transform count in advance.
+     */
+    alg_info_esp=alloc_thing (struct alg_info_esp, "alg_info_esp");
+
+    strcpy(esp_buf, alg_str);
+    if(!alg_info_discover_pfsgroup_hack(alg_info_esp, esp_buf, err_p)) {
+	return NULL;
+    }
+
+    alg_info_esp->alg_info_protoid=PROTO_IPSEC_AH;
+    ret=alg_info_parse_str((struct alg_info *)alg_info_esp
+			   , esp_buf, err_p
+			   , parser_init_ah
+			   , alg_info_ah_add
+			   , NULL
+			   , permitmann);
+
+    if (ret<0)
+	{
+	    pfreeany(alg_info_esp);
+	    alg_info_esp=NULL;
+	}
+    return alg_info_esp;
+}
+
 /*
  * 	alg_info struct can be shared by
  * 	several connections instances,
@@ -759,22 +871,37 @@ alg_info_snprint(char *buf, int buflen
 			    , (int)esp_info->esp_ealg_keylen
 			    , enum_name(&auth_alg_names, esp_info->esp_aalg_id)+sizeof("AUTH_ALGORITHM_HMAC")
 			    , esp_info->esp_aalg_id);
-		ptr+=np;
-		buflen-=np;
-		if ( cnt > 0) {
-			np=snprintf(ptr, buflen, ", ");
+		if(np < buflen) {
 			ptr+=np;
 			buflen-=np;
+		} else {
+			ptr+=buflen;
+			buflen=0;
+               }
+		if ( cnt > 0) {
+			np=snprintf(ptr, buflen, ", ");
+			if(np < buflen) {
+				ptr+=np;
+				buflen-=np;
+			} else {
+				ptr+=buflen;
+				buflen=0;
+			}
 		}
-		if(buflen<0) goto out;
+		if(buflen <= 0) goto out;
 	    }
 	    if (alg_info_esp->esp_pfsgroup) {
 		np=snprintf(ptr, buflen, "; pfsgroup=%s(%d)"
 			, enum_name(&oakley_group_names, alg_info_esp->esp_pfsgroup)+ sizeof("OAKLEY_GROUP")
 		        , alg_info_esp->esp_pfsgroup);
-		ptr+=np;
-		buflen-=np;
-		if(buflen<0) goto out;
+		if(np < buflen) {
+			ptr+=np;
+			buflen-=np;
+		} else {
+			ptr+=buflen;
+			buflen=0;
+		}
+		if(buflen <= 0) goto out;
 	    }
 	    break;
 	}
@@ -789,16 +916,26 @@ alg_info_snprint(char *buf, int buflen
 			    , (int)esp_info->esp_ealg_keylen
 			    , enum_name(&auth_alg_names, esp_info->esp_aalg_id)+sizeof("AUTH_ALGORITHM_HMAC")
 			    , esp_info->esp_aalg_id);
-		ptr+=np;
-		buflen-=np;
-		if(buflen<0) goto out;
+		if(np < buflen) {
+			ptr+=np;
+			buflen-=np;
+		} else {
+			ptr+=buflen;
+			buflen=0;
+		}
+		if(buflen <= 0) goto out;
 	    }
 	    if (alg_info_esp->esp_pfsgroup) {
 		np=snprintf(ptr, buflen, "; pfsgroup=%d; "
 			    , alg_info_esp->esp_pfsgroup);
-		ptr+=np;
-		buflen-=np;
-		if(buflen<0) goto out;
+		if(np < buflen) {
+			ptr+=np;
+			buflen-=np;
+		} else {
+			ptr+=buflen;
+			buflen=0;
+		}
+		if(buflen <= 0) goto out;
 	    }
 	    break;
         }
@@ -814,14 +951,24 @@ alg_info_snprint(char *buf, int buflen
 			    , ike_info->ike_halg
 			    , enum_name(&oakley_group_names, ike_info->ike_modp)+ sizeof("OAKLEY_GROUP")
 			    , ike_info->ike_modp);
-		ptr+=np;
-		buflen-=np;
+		if(np < buflen) {
+			ptr+=np;
+			buflen-=np;
+		} else {
+			ptr+=buflen;
+			buflen=0;
+               }
                 if ( cnt > 0) {
                         np=snprintf(ptr, buflen, ", ");
-                        ptr+=np;
-                        buflen-=np;
+			if(np < buflen) {
+				ptr+=np;
+				buflen-=np;
+			} else {
+				ptr+=buflen;
+				buflen=0;
+                        }
                 }
-		if(buflen<0) goto out;
+		if(buflen <= 0) goto out;
 	    }
 	    break;
 	}
@@ -830,15 +977,27 @@ alg_info_snprint(char *buf, int buflen
     default:
 	np=snprintf(buf, buflen, "INVALID protoid=%d\n",
 		    alg_info->alg_info_protoid);
-	ptr+=np;
-	buflen-=np;
+	if(np < buflen) {
+		ptr+=np;
+		buflen-=np;
+	} else {
+		ptr+=buflen;
+		buflen=0;
+        }
 	goto out;
     }
-    np=snprintf(ptr, buflen, "; flags=%s",
+    if(buflen > 0){
+	np=snprintf(ptr, buflen, "; flags=%s",
 		alg_info->alg_info_flags&ALG_INFO_F_STRICT?
 		"strict":"-strict");
-    ptr+=np;
-    buflen-=np;
+    	if(np < buflen) {
+		ptr+=np;
+		buflen-=np;
+    	} else {
+		ptr+=buflen;
+		buflen=0;
+    	}
+    }
 
  out:
     passert(buflen >= 0);
