@@ -11,7 +11,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: constants.c,v 1.11.2.3 2007-11-07 02:38:21 paul Exp $
  */
 
 /*
@@ -83,11 +82,12 @@ const char *const debug_bit_names[] = {
 	"private",            /* 20 */
 
 	"impair-delay-adns-key-answer", /* 21 */
-	"impair-delay-adns-txt-answer",
-	"impair-bust-mi2",
-	"impair-bust-mr2",
-	"impair-sa-creation",
-	"impair-die-oninfo",
+	"impair-delay-adns-txt-answer", /* 22 */
+	"impair-bust-mi2",   /* 23 */
+	"impair-bust-mr2",   /* 24 */
+	"impair-sa-creation", /* 25 */
+	"impair-die-oninfo",  /* 26 */
+	"impair-jacob-two-two",  /* 27 */
 
 	NULL
     };
@@ -123,7 +123,7 @@ const char *const payload_name[] = {
 	"ISAKMP_NEXT_D",
 	"ISAKMP_NEXT_VID",
 	"ISAKMP_NEXT_MODECFG",  /* 14 */
-	"ISAKMP_NEXT_15",
+	"ISAKMP_NEXT_NAT-D",
 	"ISAKMP_NEXT_16",
 	"ISAKMP_NEXT_17",
 	"ISAKMP_NEXT_18",
@@ -878,10 +878,12 @@ static const char *const rr_type_name[] = {
 	NULL
     };
 
-enum_names rr_type_names = { T_A, T_NAPTR, rr_type_name, NULL };
+enum_names rr_type_names = { ns_t_a, ns_t_naptr, rr_type_name, NULL };
 
 /* Query type values which do not appear in resource records */
 static const char *const rr_qtype_name[] = {
+    "T_TKEY",           /* 249 transaction key */
+    "TSIG",             /* 250 transaction signature */
 	"T_IXFR",	/* 251 incremental zone transfer */
 	"T_AXFR",	/* 252 transfer zone of authority */
 	"T_MAILB",	/* 253 transfer mailbox records */
@@ -890,24 +892,26 @@ static const char *const rr_qtype_name[] = {
 	NULL
     };
 
-enum_names rr_qtype_names = { T_IXFR, T_ANY, rr_qtype_name, &rr_type_names };
+enum_names rr_qtype_names = { ns_t_tkey, ns_t_any
+			      , rr_qtype_name, &rr_type_names };
 
 static const char *const rr_class_name[] = {
 	"C_IN",	/* 1 the arpa internet */
 	NULL
     };
 
-enum_names rr_class_names = { C_IN, C_IN, rr_class_name, NULL };
+enum_names rr_class_names = { ns_c_in, ns_c_in, rr_class_name, NULL };
 
 static const char *const ppk_name[] = {
   "PPK_PSK",
   "PPK_DSS",
   "PPK_RSA",
   "PPK_PIN",
+  "PPK_XAUTH",
   NULL
 };
 
-enum_names ppk_names = { PPK_PSK, PPK_PIN, ppk_name, NULL };
+enum_names ppk_names = { PPK_PSK, PPK_XAUTH, ppk_name, NULL };
 
 /*
  * NAT-Traversal defines for nat_traveral type from nat_traversal.h
@@ -943,6 +947,7 @@ struct keyword_enum_value kw_host_values[]={
     { "%opportunisticgroup", KH_OPPOGROUP },
     { "%oppogroup",     KH_OPPOGROUP },  
     { "%group",         KH_GROUP },
+    { "%hostname",      KH_IPHOSTNAME },  /* makes no sense on input */
 };
 
 struct keyword_enum_values kw_host_list=
