@@ -18,12 +18,13 @@
  * for more details.
  *
  */
-#define __NO_VERSION__
-
-#if defined (MODULE)
-#include <linux/module.h>
+#ifndef AUTOCONF_INCLUDED
+#include <linux/config.h>
 #endif
+#include <linux/version.h>
 
+#define __NO_VERSION__
+#include <linux/module.h>
 #include <linux/kernel.h> /* printk() */
 
 #include <linux/netdevice.h>   /* struct device, and other headers */
@@ -63,8 +64,8 @@
 # include "openswan/ipcomp.h"
 #endif /* CONFIG_KLIPS_COMP */
 
-#include <pfkeyv2.h>
-#include <pfkey.h>
+#include <openswan/pfkeyv2.h>
+#include <openswan/pfkey.h>
 
 #include "openswan/ipsec_alg.h"
 #include "openswan/ipsec_proto.h"
@@ -269,11 +270,9 @@ int ipsec_alg_esp_encrypt(struct ipsec_sa *sa_p, __u8 * idat,
 		    "entering with encalg=%d, ixt_e=%p\n",
 		    sa_p->ips_encalg, ixt_e);
 	if (ixt_e == NULL) {
-#ifdef CONFIG_KLIPS_DEBUG
 	  KLIPS_ERROR(debug_flag,
 		      "klips_debug:ipsec_alg_esp_encrypt: "
 		      "NULL ipsec_alg_enc object\n");
-#endif
 		return -1;
 	}
 	KLIPS_PRINT(debug_flag,
@@ -851,7 +850,7 @@ int ipsec_alg_init(void) {
 	}
 #endif
 
-#if defined(CONFIG_KLIPS_ENC_3DES) && CONFIG_KLIPS_ENC_3DES && !defined(CONFIG_KLIPS_ENC_3DES_MODULE) 
+#if defined(CONFIG_KLIPS_ENC_3DES) && !defined(CONFIG_KLIPS_ENC_3DES_MODULE) 
 #if defined(CONFIG_KLIPS_ENC_CRYPTOAPI) && CONFIG_KLIPS_ENC_CRYPTOAPI
 #warning "Using built-in 3des rather than CryptoAPI 3des"
 #endif	
@@ -860,17 +859,6 @@ int ipsec_alg_init(void) {
 		ipsec_3des_init();
 	}
 #endif
-#if defined(CONFIG_KLIPS_ENC_NULL) && CONFIG_KLIPS_ENC_NULL && !defined(CONFIG_KLIPS_ENC_NULL_MODULE) 
-#if defined(CONFIG_KLIPS_ENC_CRYPTOAPI) && CONFIG_KLIPS_ENC_CRYPTOAPI
-#warning "Using built-in null cipher rather than CryptoAPI null cipher"
-#endif	
-#warning "Building with null cipher (ESP_NULL), blame on you :-)"
-	{
-		extern int ipsec_null_init(void);
-		ipsec_null_init();
-	}
-#endif
-
 
 	/* If we are doing CryptoAPI, then init */
 #if defined(CONFIG_KLIPS_ENC_CRYPTOAPI) && CONFIG_KLIPS_ENC_CRYPTOAPI && !defined(CONFIG_KLIPS_ENC_CRYPTOAPI_MODULE)
