@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: adns.c,v 1.11.24.1 2007-09-05 03:01:04 paul Exp $
+ * RCSID $Id: adns.c,v 1.11.24.1 2007/09/05 03:01:04 paul Exp $
  */
 
 #ifndef USE_LWRES	/* whole file! */
@@ -233,14 +233,14 @@ worker(int qfd, int afd)
 	a.amagic = ADNS_A_MAGIC;
 	a.serial = q.serial;
 
-	a.result = res_nquery(statp, q.name_buf, C_IN, q.type, a.ans, sizeof(a.ans));
+	a.result = res_nquery(statp, q.name_buf, ns_c_in, q.type, a.ans, sizeof(a.ans));
 	a.h_errno_val = h_errno;
 
 	a.len = offsetof(struct adns_answer, ans) + (a.result < 0? 0 : a.result);
 
 #ifdef DEBUG
-	if (((q.debugging & IMPAIR_DELAY_ADNS_KEY_ANSWER) && q.type == T_KEY)
-	|| ((q.debugging & IMPAIR_DELAY_ADNS_TXT_ANSWER) && q.type == T_TXT))
+	if (((q.debugging & IMPAIR_DELAY_ADNS_KEY_ANSWER) && q.type == ns_t_key)
+	|| ((q.debugging & IMPAIR_DELAY_ADNS_TXT_ANSWER) && q.type == ns_t_txt))
 	    sleep(30);	/* delay the answer */
 #endif
 
@@ -504,11 +504,11 @@ answer(struct worker_info *w)
     else
     {
 	/* pass the answer on to Pluto */
-	enum helper_exit_status r
+	enum helper_exit_status rs
 	    = write_pipe(PLUTO_AFD, (const unsigned char *) &a);
 
-	if (r != HES_CONTINUE)
-	    exit(r);
+	if (rs != HES_CONTINUE)
+	    exit(rs);
 	w->busy = FALSE;
 	forward_query(w);
     }
