@@ -1,5 +1,5 @@
 /* 
- * Cryptographic helper function - calculate KE and nonce
+ * Cryptographic helper function - calculate DH
  * Copyright (C) 2004 Michael C. Richardson <mcr@xelerance.com>
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -14,7 +14,7 @@
  *
  * This code was developed with the support of IXIA communications.
  *
- * RCSID $Id: crypt_dh.c,v 1.8.2.1 2006-03-20 13:32:03 paul Exp $
+ * RCSID $Id: crypt_dh.c,v 1.11 2005/08/14 21:47:29 mcr Exp $
  */
 
 #include <stdlib.h>
@@ -25,7 +25,6 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <unistd.h>
-#include <sys/queue.h>
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <sys/types.h>
@@ -34,6 +33,7 @@
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
 
+#include "sysdep.h"
 #include "constants.h"
 #include "defs.h"
 #include "packet.h"
@@ -46,7 +46,9 @@
 #include "log.h"
 #include "timer.h"
 #include "ike_alg.h"
+#include "id.h"
 #include "secrets.h"
+#include "keys.h"
 
 /** Compute DH shared secret from our local secret and the peer's public value.
  * We make the leap that the length should be that of the group
@@ -85,7 +87,7 @@ calc_dh_shared(chunk_t *shared, const chunk_t g
 		, tv_diff);
     }
 
-    DBG_cond_dump_chunk(DBG_CRYPT, "DH shared secret:\n", *shared);
+    DBG_cond_dump_chunk(DBG_CRYPT, "DH shared-secret:\n", *shared);
 }
 
 
@@ -128,7 +130,7 @@ skeyid_digisig(const chunk_t ni
 
     DBG(DBG_CRYPT,
 	DBG_log("skeyid inputs (digi+NI+NR+shared) hasher: %s", hasher->common.name);
-	DBG_dump_chunk("shared: ", shared);
+	DBG_dump_chunk("shared-secret: ", shared);
 	DBG_dump_chunk("ni: ", ni);
 	DBG_dump_chunk("nr: ", nr));
     
@@ -216,7 +218,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
 	hmac_update_chunk(&ctx, shared);
 	hmac_update_chunk(&ctx, icookie);
 	hmac_update_chunk(&ctx, rcookie);
-	hmac_update(&ctx, "\0", 1);
+	hmac_update(&ctx, (const u_char *)"\0", 1);
 	hmac_final_chunk(*skeyid_d, "st_skeyid_d in generate_skeyids_iv()", &ctx);
 
 	/* SKEYID_A */
@@ -225,7 +227,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
 	hmac_update_chunk(&ctx, shared);
 	hmac_update_chunk(&ctx, icookie);
 	hmac_update_chunk(&ctx, rcookie);
-	hmac_update(&ctx, "\1", 1);
+	hmac_update(&ctx, (const u_char *)"\1", 1);
 	hmac_final_chunk(*skeyid_a, "st_skeyid_a in generate_skeyids_iv()", &ctx);
 
 	/* SKEYID_E */
@@ -234,7 +236,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
 	hmac_update_chunk(&ctx, shared);
 	hmac_update_chunk(&ctx, icookie);
 	hmac_update_chunk(&ctx, rcookie);
-	hmac_update(&ctx, "\2", 1);
+	hmac_update(&ctx, (const u_char *)"\2", 1);
 	hmac_final_chunk(*skeyid_e, "st_skeyid_e in generate_skeyids_iv()", &ctx);
     }
 
@@ -270,7 +272,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
 	    size_t i = 0;
 
 	    hmac_init_chunk(&ctx, hasher, *skeyid_e);
-	    hmac_update(&ctx, "\0", 1);
+	    hmac_update(&ctx, (const u_char *)"\0", 1);
 	    for (;;)
 	    {
 		hmac_final(&keytemp[i], &ctx);
@@ -302,7 +304,7 @@ void calc_dh_iv(struct pluto_crypto_req *r)
     struct pcr_skeyid_q dhq;
     const struct oakley_group_desc *group;
     MP_INT  sec;
-    chunk_t  shared, g;
+    chunk_t  shared, g, ltsecret;
     chunk_t  skeyid, skeyid_d, skeyid_a, skeyid_e; 
     chunk_t  new_iv, enc_key;
 
@@ -315,6 +317,7 @@ void calc_dh_iv(struct pluto_crypto_req *r)
     skr->thespace.len   = sizeof(skr->space);
 
     group = lookup_group(dhq.oakley_group);
+    passert(group != NULL);
 
     pluto_crypto_allocchunk(&skr->thespace
 			   , &skr->shared
@@ -322,8 +325,14 @@ void calc_dh_iv(struct pluto_crypto_req *r)
     shared.ptr = wire_chunk_ptr(skr, &skr->shared);
     shared.len = group->bytes;
 
+    ltsecret.ptr = wire_chunk_ptr(&dhq, &dhq.secret);
+    ltsecret.len = dhq.secret.len;
+
     /* recover the long term secret */
-    n_to_mpz(&sec, wire_chunk_ptr(&dhq, &dhq.secret), dhq.secret.len);
+    n_to_mpz(&sec, ltsecret.ptr, ltsecret.len);
+
+    DBG(DBG_CRYPT,
+	DBG_dump_chunk("long term secret: ", ltsecret));
 
     /* now calculate the (g^x)(g^y) --- need gi on responder, gr on initiator */
 
@@ -334,7 +343,6 @@ void calc_dh_iv(struct pluto_crypto_req *r)
     }
 
     calc_dh_shared(&shared, g, &sec, group);
-    mpz_clear (&sec);
     
     memset(&skeyid, 0, sizeof(skeyid));
     memset(&skeyid_d, 0, sizeof(skeyid_d));
@@ -409,7 +417,6 @@ void calc_dh(struct pluto_crypto_req *r)
       setchunk_fromwire(g, &dhq.gr, &dhq);
     }
     calc_dh_shared(&shared, g, &sec, group);
-    mpz_clear (&sec);
 
     /* now translate it back to wire chunks, freeing the chunks */
     setwirechunk_fromchunk(skr->shared,   shared,   skr);
@@ -419,116 +426,6 @@ void calc_dh(struct pluto_crypto_req *r)
 }
 
 
-stf_status perform_dh_secretiv(struct state *st
-			     , enum phase1_role init       /* TRUE=g_init,FALSE=g_r */
-			     , u_int16_t oakley_group)
-{
-    struct pluto_crypto_req r;
-    struct pcr_skeyid_q *dhq = &r.pcr_d.dhq;
-    struct pcr_skeyid_r *dhr = &r.pcr_d.dhr;
-    const chunk_t *pss = get_preshared_secret(st->st_connection);
-
-    passert(st->st_sec_in_use);
-
-    dhq->thespace.start = 0;
-    dhq->thespace.len   = sizeof(dhq->space);
-
-    /* convert appropriate data to dhq */
-    dhq->auth = st->st_oakley.auth;
-    dhq->hash = st->st_oakley.hash;
-    dhq->oakley_group = oakley_group;
-    dhq->init = init;
-    dhq->keysize = st->st_oakley.enckeylen/BITS_PER_BYTE; 
-
-    if(pss) {
-	pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->pss, *pss);
-    }
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->ni,  st->st_ni);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->nr,  st->st_nr);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->gi,  st->st_gi);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->gr,  st->st_gr);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space
-			   , &dhq->secret, st->st_sec_chunk);
-
-    pluto_crypto_allocchunk(&dhq->thespace, &dhq->icookie, COOKIE_SIZE);
-    memcpy(wire_chunk_ptr(&r.pcr_d.dhq, &dhq->icookie)
-	   , st->st_icookie, COOKIE_SIZE);
-
-    pluto_crypto_allocchunk(&dhq->thespace, &dhq->rcookie, COOKIE_SIZE);
-    memcpy(wire_chunk_ptr(&r.pcr_d.dhq, &dhq->rcookie)
-	   , st->st_rcookie, COOKIE_SIZE);
-
-    calc_dh_iv(&r);
-
-    clonetochunk(st->st_shared,   wire_chunk_ptr(dhr, &(dhr->shared))
-		 , dhr->shared.len,   "calculated shared secret");
-    clonetochunk(st->st_skeyid,   wire_chunk_ptr(dhr, &(dhr->skeyid))
-		 , dhr->skeyid.len,   "calculated skeyid secret");
-    clonetochunk(st->st_skeyid_d, wire_chunk_ptr(dhr, &(dhr->skeyid_d))
-		 , dhr->skeyid_d.len, "calculated skeyid_d secret");
-    clonetochunk(st->st_skeyid_a, wire_chunk_ptr(dhr, &(dhr->skeyid_a))
-		 , dhr->skeyid_a.len, "calculated skeyid_a secret");
-    clonetochunk(st->st_skeyid_e, wire_chunk_ptr(dhr, &(dhr->skeyid_e))
-		 , dhr->skeyid_e.len, "calculated skeyid_a secret");
-    clonetochunk(st->st_enc_key, wire_chunk_ptr(dhr, &(dhr->enc_key))
-		 , dhr->enc_key.len, "calculated key for phase 1");
-    
-    passert(dhr->new_iv.len <= MAX_DIGEST_LEN);
-    passert(dhr->new_iv.len > 0);
-    memcpy(st->st_new_iv, wire_chunk_ptr(dhr, &(dhr->new_iv)),dhr->new_iv.len);
-    st->st_new_iv_len = dhr->new_iv.len;
-
-    st->hidden_variables.st_skeyid_calculated = TRUE;
-    return STF_OK;
-}
-
-stf_status perform_dh_secret(struct state *st
-			     , enum phase1_role init      
-			     , u_int16_t oakley_group)
-{
-    struct pluto_crypto_req r;
-    struct pcr_skeyid_q *dhq = &r.pcr_d.dhq;
-    struct pcr_skeyid_r *dhr = &r.pcr_d.dhr;
-    const chunk_t *pss = get_preshared_secret(st->st_connection);
-
-    passert(st->st_sec_in_use);
-
-    dhq->thespace.start = 0;
-    dhq->thespace.len   = sizeof(dhq->space);
-
-    /* convert appropriate data to dhq */
-    dhq->auth = st->st_oakley.auth;
-    dhq->hash = st->st_oakley.hash;
-    dhq->oakley_group = oakley_group;
-    dhq->init = init;
-    dhq->keysize = st->st_oakley.enckeylen/BITS_PER_BYTE; 
-
-    if(pss) {
-	pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->pss, *pss);
-    }
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->ni,  st->st_ni);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->nr,  st->st_nr);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->gi,  st->st_gi);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space, &dhq->gr,  st->st_gr);
-    pluto_crypto_copychunk(&dhq->thespace, dhq->space
-			   , &dhq->secret, st->st_sec_chunk);
-
-    pluto_crypto_allocchunk(&dhq->thespace, &dhq->icookie, COOKIE_SIZE);
-    memcpy(wire_chunk_ptr(&r.pcr_d.dhq, &dhq->icookie)
-	   , st->st_icookie, COOKIE_SIZE);
-
-    pluto_crypto_allocchunk(&dhq->thespace, &dhq->rcookie, COOKIE_SIZE);
-    memcpy(wire_chunk_ptr(&r.pcr_d.dhq, &dhq->rcookie)
-	   , st->st_rcookie, COOKIE_SIZE);
-
-    calc_dh(&r);
-
-    clonetochunk(st->st_shared,   wire_chunk_ptr(dhr, &(dhr->shared))
-		 , dhr->shared.len,   "calculated shared secret");
-    
-    return STF_OK;
-}
-
 /*
  * Local Variables:
  * c-basic-offset:4
