@@ -15,7 +15,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ikev1_quick.c,v 1.3.2.9 2008-02-14 22:22:34 paul Exp $
  */
 
 #include <stdio.h>
@@ -26,14 +25,13 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <resolv.h>
 #include <arpa/nameser.h>	/* missing from <resolv.h> on old systems */
-#include <sys/queue.h>
 #include <sys/time.h>		/* for gettimeofday */
 
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
 
+#include "sysdep.h"
 #include "constants.h"
 #include "defs.h"
 #include "state.h"
@@ -84,11 +82,10 @@
 #ifdef NAT_TRAVERSAL
 #include "nat_traversal.h"
 #endif
-#ifdef VIRTUAL_IP
 #include "virtual.h"
-#endif
 #include "dpd.h"
 #include "x509more.h"
+#include "tpm/tpm.h"
 
 /* accept_PFS_KE
  *
@@ -195,8 +192,9 @@ emit_subnet_id(ip_subnet *net
  */
 static void
 compute_proto_keymat(struct state *st
-, u_int8_t protoid
-, struct ipsec_proto_info *pi)
+		     , u_int8_t protoid
+		     , struct ipsec_proto_info *pi
+		     , const char *satypename)
 {
     size_t needed_len; /* bytes of keying material needed */
 
@@ -282,6 +280,12 @@ compute_proto_keymat(struct state *st
 		needed_len = HMAC_SHA1_KEY_LEN;
 		break;
 	    default:
+#ifdef KERNEL_ALG
+		if (kernel_alg_ah_auth_ok(pi->attrs.auth, NULL)) {
+		    needed_len += kernel_alg_ah_auth_keylen(pi->attrs.auth);
+		    break;
+		} 
+#endif
 		bad_case(pi->attrs.transid);
 	    }
 	    break;
@@ -349,17 +353,18 @@ compute_proto_keymat(struct state *st
     }
 
     DBG(DBG_CRYPT,
-	DBG_dump("KEYMAT computed:\n", pi->our_keymat, pi->keymat_len);
-	DBG_dump("Peer KEYMAT computed:\n", pi->peer_keymat, pi->keymat_len));
+	DBG_log("%s KEYMAT\n",satypename);
+	DBG_dump("  KEYMAT computed:\n", pi->our_keymat, pi->keymat_len);
+	DBG_dump("  Peer KEYMAT computed:\n", pi->peer_keymat, pi->keymat_len));
 }
 
 static void
 compute_keymats(struct state *st)
 {
     if (st->st_ah.present)
-	compute_proto_keymat(st, PROTO_IPSEC_AH, &st->st_ah);
+	compute_proto_keymat(st, PROTO_IPSEC_AH, &st->st_ah, "AH");
     if (st->st_esp.present)
-	compute_proto_keymat(st, PROTO_IPSEC_ESP, &st->st_esp);
+	compute_proto_keymat(st, PROTO_IPSEC_ESP, &st->st_esp, "ESP");
 }
 
 /* Decode the variable part of an ID packet (during Quick Mode).
@@ -394,7 +399,9 @@ decode_net_id(struct isakmp_ipsec_id *id
 	    afi = &af_inet6_info;
 	    break;
 	case ID_FQDN:
-		return TRUE;
+	    loglog(RC_COMMENT, "%s type is FQDN", which);
+	    return TRUE;
+
 	default:
 	    /* XXX support more */
 	    loglog(RC_LOG_SERIOUS, "unsupported ID type %s"
@@ -409,14 +416,14 @@ decode_net_id(struct isakmp_ipsec_id *id
 	case ID_IPV6_ADDR:
 	{
 	    ip_address temp_address;
-	    err_t ugh;
+	    err_t ughmsg;
 
-	    ugh = initaddr(id_pbs->cur, pbs_left(id_pbs), afi->af, &temp_address);
+	    ughmsg = initaddr(id_pbs->cur, pbs_left(id_pbs), afi->af, &temp_address);
 
-	    if (ugh != NULL)
+	    if (ughmsg != NULL)
 	    {
 		loglog(RC_LOG_SERIOUS, "%s ID payload %s has wrong length in Quick I1 (%s)"
-		    , which, idtypename, ugh);
+		    , which, idtypename, ughmsg);
 		/* XXX Could send notification back */
 		return FALSE;
 	    }
@@ -437,7 +444,7 @@ decode_net_id(struct isakmp_ipsec_id *id
 	case ID_IPV6_ADDR_SUBNET:
 	{
 	    ip_address temp_address, temp_mask;
-	    err_t ugh;
+	    err_t ughmsg;
 
 	    if (pbs_left(id_pbs) != 2 * afi->ia_sz)
 	    {
@@ -446,20 +453,20 @@ decode_net_id(struct isakmp_ipsec_id *id
 		/* XXX Could send notification back */
 		return FALSE;
 	    }
-	    ugh = initaddr(id_pbs->cur
+	    ughmsg = initaddr(id_pbs->cur
 		, afi->ia_sz, afi->af, &temp_address);
-	    if (ugh == NULL)
-		ugh = initaddr(id_pbs->cur + afi->ia_sz
+	    if (ughmsg == NULL)
+		ughmsg = initaddr(id_pbs->cur + afi->ia_sz
 		    , afi->ia_sz, afi->af, &temp_mask);
-	    if (ugh == NULL)
-		ugh = initsubnet(&temp_address, masktocount(&temp_mask)
+	    if (ughmsg == NULL)
+		ughmsg = initsubnet(&temp_address, masktocount(&temp_mask)
 		    , '0', net);
-	    if (ugh == NULL && subnetisnone(net))
-		ugh = "contains only anyaddr";
-	    if (ugh != NULL)
+	    if (ughmsg == NULL && subnetisnone(net))
+		ughmsg = "contains only anyaddr";
+	    if (ughmsg != NULL)
 	    {
 		loglog(RC_LOG_SERIOUS, "%s ID payload %s bad subnet in Quick I1 (%s)"
-		    , which, idtypename, ugh);
+		    , which, idtypename, ughmsg);
 		/* XXX Could send notification back */
 		return FALSE;
 	    }
@@ -477,7 +484,7 @@ decode_net_id(struct isakmp_ipsec_id *id
 	case ID_IPV6_ADDR_RANGE:
 	{
 	    ip_address temp_address_from, temp_address_to;
-	    err_t ugh;
+	    err_t ughmsg;
 
 	    if (pbs_left(id_pbs) != 2 * afi->ia_sz)
 	    {
@@ -486,22 +493,22 @@ decode_net_id(struct isakmp_ipsec_id *id
 		/* XXX Could send notification back */
 		return FALSE;
 	    }
-	    ugh = initaddr(id_pbs->cur, afi->ia_sz, afi->af, &temp_address_from);
-	    if (ugh == NULL)
-		ugh = initaddr(id_pbs->cur + afi->ia_sz
+	    ughmsg = initaddr(id_pbs->cur, afi->ia_sz, afi->af, &temp_address_from);
+	    if (ughmsg == NULL)
+		ughmsg = initaddr(id_pbs->cur + afi->ia_sz
 		    , afi->ia_sz, afi->af, &temp_address_to);
-	    if (ugh != NULL)
+	    if (ughmsg != NULL)
 	    {
 		loglog(RC_LOG_SERIOUS, "%s ID payload %s malformed (%s) in Quick I1"
-		    , which, idtypename, ugh);
+		    , which, idtypename, ughmsg);
 		/* XXX Could send notification back */
 		return FALSE;
 	    }
 
-	    ugh = rangetosubnet(&temp_address_from, &temp_address_to, net);
-	    if (ugh == NULL && subnetisnone(net))
-		ugh = "contains only anyaddr";
-	    if (ugh != NULL)
+	    ughmsg = rangetosubnet(&temp_address_from, &temp_address_to, net);
+	    if (ughmsg == NULL && subnetisnone(net))
+		ughmsg = "contains only anyaddr";
+	    if (ughmsg != NULL)
 	    {
 		char temp_buff1[ADDRTOT_BUF], temp_buff2[ADDRTOT_BUF];
 
@@ -509,7 +516,7 @@ decode_net_id(struct isakmp_ipsec_id *id
 		addrtot(&temp_address_to, 0, temp_buff2, sizeof(temp_buff2));
 		loglog(RC_LOG_SERIOUS, "%s ID payload in Quick I1, %s"
 		    " %s - %s unacceptable: %s"
-		    , which, idtypename, temp_buff1, temp_buff2, ugh);
+		    , which, idtypename, temp_buff1, temp_buff2, ughmsg);
 		return FALSE;
 	    }
 	    DBG(DBG_PARSING | DBG_CONTROL,
@@ -538,11 +545,11 @@ decode_net_id(struct isakmp_ipsec_id *id
 /* like decode, but checks that what is received matches what was sent */
 static bool
 check_net_id(struct isakmp_ipsec_id *id
-, pb_stream *id_pbs
-, u_int8_t *protoid
-, u_int16_t *port
-, ip_subnet *net
-, const char *which)
+	     , pb_stream *id_pbs
+	     , u_int8_t *protoid
+	     , u_int16_t *port
+	     , ip_subnet *net
+	     , const char *which)
 {
     ip_subnet net_temp;
     bool bad_proposal=FALSE;
@@ -565,8 +572,8 @@ check_net_id(struct isakmp_ipsec_id *id
 #endif
     }
     if(*protoid != id->isaiid_protoid) {
-	loglog(RC_LOG_SERIOUS, "%s peer returned protocol id doesn't match my proposal - us%d vs them: %d",
-		 which,*protoid,id->isaiid_protoid);
+	loglog(RC_LOG_SERIOUS, "%s peer returned protocol id does not match my proposal - us%d vs them: %d"
+		, which, *protoid, id->isaiid_protoid);
 #ifdef ALLOW_MICROSOFT_BAD_PROPOSAL
 	loglog(RC_LOG_SERIOUS, "Allowing questionable proposal anyway [ALLOW_MICROSOFT_BAD_PROPOSAL]");
 	bad_proposal = FALSE;
@@ -578,7 +585,7 @@ check_net_id(struct isakmp_ipsec_id *id
      * workaround for #802- "our client ID returned doesn't match my proposal"
      * until such time as bug #849 is properly fixed.
      */
-    if (*port != id->isaiid_port) { 
+    if (*port != id->isaiid_port) {
 	loglog(RC_LOG_SERIOUS, "%s peer returned port doesn't match my proposal - us:%d vs them:%d",
 		which,*port,id->isaiid_port );
 	if (*port != 0 && id->isaiid_port!=1701) {
@@ -637,7 +644,7 @@ quick_mode_hash3(u_char *dest, struct state *st)
     struct hmac_ctx ctx;
 
     hmac_init_chunk(&ctx, st->st_oakley.hasher, st->st_skeyid_a);
-    hmac_update(&ctx, "\0", 1);
+    hmac_update(&ctx, (const u_char *)"\0", 1);
     hmac_update(&ctx, (u_char *) &st->st_msgid, sizeof(st->st_msgid));
     hmac_update_chunk(&ctx, st->st_ni);
     hmac_update_chunk(&ctx, st->st_nr);
@@ -698,6 +705,7 @@ quick_outI1_continue(struct pluto_crypto_req_cont *pcrc
     passert(st != NULL);
 
     set_cur_state(st);	/* we must reset before exit */
+    set_suspended(st, NULL);
     e = quick_outI1_tail(pcrc, r);
 
     reset_globals();
@@ -712,8 +720,11 @@ quick_outI1(int whack_sock
 	    , so_serial_t replacing)
 {
     struct state *st = duplicate_state(isakmp_sa);
-    struct qke_continuation *qke;
+    struct qke_continuation *qke = alloc_thing(struct qke_continuation
+					       , "quick_outI1 KE");
     stf_status e;
+    const char *pfsgroupname;
+    char p2alg[256];
 
     st->st_whack_sock = whack_sock;
     st->st_connection = c;
@@ -737,33 +748,47 @@ quick_outI1(int whack_sock
 
     insert_state(st);	/* needs cookies, connection, and msgid */
 
-    if (replacing == SOS_NOBODY)
-	openswan_log("initiating Quick Mode %s {using isakmp#%lu}"
-		     , prettypolicy(policy)
-		     , isakmp_sa->st_serialno);
-    else
-	openswan_log("initiating Quick Mode %s to replace #%lu {using isakmp#%lu}"
-		     , prettypolicy(policy)
-		     , replacing
-		     , isakmp_sa->st_serialno);
+    strcpy(p2alg, "defaults");
+    if(st->st_connection->alg_info_esp) {
+	alg_info_snprint_phase2(p2alg, sizeof(p2alg)
+				, (struct alg_info_esp *)st->st_connection->alg_info_esp);
+    }
 
+    pfsgroupname="no-pfs";
     /* 
      * See if pfs_group has been specified for this conn,
      * if not, fallback to old use-same-as-P1 behaviour
      */
-#ifdef IKE_ALG
-    if (st->st_connection)
-	    st->st_pfs_group = ike_alg_pfsgroup(st->st_connection
-						, st->st_policy);
-    if (!st->st_pfs_group)
-#endif
+    if (st->st_connection) {
+	st->st_pfs_group = ike_alg_pfsgroup(st->st_connection
+					    , st->st_policy);
+	
+    }
+
     /* If PFS specified, use the same group as during Phase 1:
      * since no negotiation is possible, we pick one that is
      * very likely supported.
      */
+    if (!st->st_pfs_group)
 	    st->st_pfs_group = policy & POLICY_PFS? isakmp_sa->st_oakley.group : NULL;
 
-    qke = alloc_thing(struct qke_continuation, "quick_outI1 KE");
+    if(policy & POLICY_PFS && st->st_pfs_group) {
+	pfsgroupname = enum_name(&oakley_group_names, st->st_pfs_group->group);
+    }
+
+    {
+	char replacestr[32];
+
+	replacestr[0]='\0';
+	if(replacing != SOS_NOBODY)
+	    snprintf(replacestr, 32, " to replace #%lu", replacing);
+	
+	openswan_log("initiating Quick Mode %s%s {using isakmp#%lu msgid:%08x proposal=%s pfsgroup=%s}"
+		     , prettypolicy(policy)
+		     , replacestr
+		     , isakmp_sa->st_serialno, st->st_msgid, p2alg, pfsgroupname);
+    }
+
     qke->st = st;
     qke->isakmp_sa = isakmp_sa;
     qke->replacing = replacing;
@@ -915,13 +940,23 @@ quick_outI1_tail(struct pluto_crypto_req_cont *pcrc
 	&& (!(st->st_policy & POLICY_TUNNEL))
 	&& (st->hidden_variables.st_nat_traversal & LELEM(NAT_TRAVERSAL_NAT_BHND_ME))) {
 	/** Send NAT-OA if our address is NATed */
-	if (!nat_traversal_add_natoa(ISAKMP_NEXT_NONE, &rbody, st)) {
+	if (!nat_traversal_add_natoa(ISAKMP_NEXT_NONE, &rbody, st, TRUE /* initiator */)) {
 	    reset_cur_state();
 	    return STF_INTERNAL_ERROR;
 	}
     }
 #endif
 
+#ifdef TPM
+    {
+	pb_stream *pbs = &rbody;
+	size_t enc_len = pbs_offset(pbs) - sizeof(struct isakmp_hdr);
+
+	TCLCALLOUT_crypt("preHash",st,pbs,sizeof(struct isakmp_hdr),enc_len);
+	r_hashval = tpm_relocateHash(pbs);
+    }
+#endif
+
     /* finish computing  HASH(1), inserting it in output */
     (void) quick_mode_hash12(r_hashval, r_hash_start, rbody.cur
 	, st, &st->st_msgid, FALSE);
@@ -993,6 +1028,15 @@ quick_outI1_tail(struct pluto_crypto_req_cont *pcrc
  * consulted asynchronously, gateways_from_dns != NULL the second time.
  * Remember that our state object might disappear too!
  *
+ * At the end of authtail, we have all the info we need, but we
+ * haven't done any nonce generation or DH that we might need
+ * to do, so that are two crypto continuations that do this work,
+ * they are:
+ *    quick_inI1_outR1_cryptocontinue1 -- called after NONCE/KE
+ *    quick_inI1_outR1_cryptocontinue2 -- called after DH (if PFS)
+ *
+ * we have to call nonce/ke and DH if we are doing PFS.
+ *
  *
  * If the connection is opportunistic, we must verify delegation.
  *
@@ -1098,21 +1142,22 @@ quick_inI1_outR1(struct msg_digest *md)
 
     if (id_pd != NULL)
     {
+	struct payload_digest *IDci = id_pd->next;
+
 	/* ??? we are assuming IPSEC_DOI */
 
 	/* IDci (initiator is peer) */
 
 	if (!decode_net_id(&id_pd->payload.ipsec_id, &id_pd->pbs
-	, &b.his.net, "peer client"))
+			   , &b.his.net, "peer client"))
 	    return STF_FAIL + INVALID_ID_INFORMATION;
 
         /* Hack for MS 818043 NAT-T Update */
-        if (id_pd->payload.ipsec_id.isaiid_idtype == ID_FQDN)
-           memset(&b.his.net, 0, sizeof(ip_subnet));
-
-        if (id_pd->payload.ipsec_id.isaiid_idtype == ID_FQDN)
-           happy(addrtosubnet(&c->spd.that.host_addr, &b.his.net));
-
+        if (id_pd->payload.ipsec_id.isaiid_idtype == ID_FQDN) {
+	    loglog(RC_LOG_SERIOUS, "Applying workaround for MS-818043 NAT-T bug");
+	    memset(&b.his.net, 0, sizeof(ip_subnet));
+	    happy(addrtosubnet(&c->spd.that.host_addr, &b.his.net));
+	}
 	/* End Hack for MS 818043 NAT-T Update */
 
 	b.his.proto = id_pd->payload.ipsec_id.isaiid_protoid;
@@ -1121,13 +1166,45 @@ quick_inI1_outR1(struct msg_digest *md)
 
 	/* IDcr (we are responder) */
 
-	if (!decode_net_id(&id_pd->next->payload.ipsec_id, &id_pd->next->pbs
-	, &b.my.net, "our client"))
+	if (!decode_net_id(&IDci->payload.ipsec_id, &IDci->pbs
+			   , &b.my.net, "our client"))
 	    return STF_FAIL + INVALID_ID_INFORMATION;
 
-	b.my.proto = id_pd->next->payload.ipsec_id.isaiid_protoid;
-	b.my.port = id_pd->next->payload.ipsec_id.isaiid_port;
+	b.my.proto = IDci->payload.ipsec_id.isaiid_protoid;
+	b.my.port = IDci->payload.ipsec_id.isaiid_port;
 	b.my.net.addr.u.v4.sin_port = htons(b.my.port);
+
+#ifdef NAT_TRAVERSAL
+	/*
+	 * if there is a NATOA payload, then use it as
+	 *    &st->st_connection->spd.that.client, if the type
+	 * of the ID was FQDN
+	 *
+	 * we actually do NATOA calculation again later on,
+	 * but we need the info here, and we don't have a state
+	 * to store it in until after we've done the authorization steps.
+	 */
+	if ((p1st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) &&
+	    (p1st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA) &&
+	    (id_pd->payload.ipsec_id.isaiid_idtype == ID_FQDN)) {
+	    struct hidden_variables hv; 
+	    char idfqdn[32], subnet_buf[SUBNETTOT_BUF]; 
+	    int  idlen = pbs_room(&IDci->pbs);
+
+	    if(idlen > 31) idlen=31;
+	    memcpy(idfqdn, IDci->pbs.cur, idlen);
+	    idfqdn[idlen]='\0';
+
+	    hv = p1st->hidden_variables; 
+	    nat_traversal_natoa_lookup(md, &hv); 
+	    
+	    addrtosubnet(&hv.st_nat_oa,&b.his.net);
+	    subnettot(&b.his.net, 0, subnet_buf, sizeof(subnet_buf));
+
+	    loglog(RC_LOG_SERIOUS, "IDci was FQDN: %s, using NAT_OA=%s as IDci"
+		   , idfqdn, subnet_buf);
+	}
+#endif
     }
     else
     {
@@ -1202,7 +1279,7 @@ quick_inI1_outR1_continue(struct adns_continuation *cr, err_t ugh)
     if (st != NULL)
     {
 	passert(st->st_suspended_md == b->md);
-	st->st_suspended_md = NULL;	/* no longer connected or suspended */
+	set_suspended(st, NULL);	/* no longer connected or suspended */
 	cur_state = st;
 	if (!b->failure_ok && ugh != NULL)
 	{
@@ -1229,8 +1306,8 @@ quick_inI1_outR1_start_query(struct verify_oppo_bundle *b
     struct connection *c = p1st->st_connection;
     struct verify_oppo_continuation *vc
 	= alloc_thing(struct verify_oppo_continuation, "verify continuation");
+    const struct id *our_id;
     struct id id	/* subject of query */
-	, *our_id	/* needed for myid playing */
 	, our_id_space;	/* ephemeral: no need for unshare_id_content */
     ip_address client;
     err_t ugh;
@@ -1239,7 +1316,7 @@ quick_inI1_outR1_start_query(struct verify_oppo_bundle *b
     b->step = next_step;    /* not just vc->b.step */
     vc->b = *b;
     passert(p1st->st_suspended_md == NULL);
-    p1st->st_suspended_md = b->md;
+    set_suspended(p1st, b->md);
 
     DBG(DBG_CONTROL,
 	{
@@ -1282,7 +1359,7 @@ quick_inI1_outR1_start_query(struct verify_oppo_bundle *b
 	vc->b.failure_ok = b->failure_ok = FALSE;
 	ugh = start_adns_query(&id
 	    , our_id
-	    , T_TXT
+	    , ns_t_txt
 	    , quick_inI1_outR1_continue
 	    , &vc->ac);
 	break;
@@ -1291,7 +1368,7 @@ quick_inI1_outR1_start_query(struct verify_oppo_bundle *b
 	vc->b.failure_ok = b->failure_ok = TRUE;
 	ugh = start_adns_query(our_id
 	    , our_id	/* self as SG */
-	    , T_TXT
+	    , ns_t_txt
 	    , quick_inI1_outR1_continue
 	    , &vc->ac);
 	break;
@@ -1301,7 +1378,7 @@ quick_inI1_outR1_start_query(struct verify_oppo_bundle *b
 	vc->b.failure_ok = b->failure_ok = FALSE;
 	ugh = start_adns_query(our_id
 	    , NULL
-	    , T_KEY
+	    , ns_t_key
 	    , quick_inI1_outR1_continue
 	    , &vc->ac);
 	break;
@@ -1313,7 +1390,7 @@ quick_inI1_outR1_start_query(struct verify_oppo_bundle *b
 	vc->b.failure_ok = b->failure_ok = FALSE;
 	ugh = start_adns_query(&id
 	    , &c->spd.that.id
-	    , T_TXT
+	    , ns_t_txt
 	    , quick_inI1_outR1_continue
 	    , &vc->ac);
 	break;
@@ -1325,11 +1402,13 @@ quick_inI1_outR1_start_query(struct verify_oppo_bundle *b
     if (ugh != NULL)
     {
 	/* note: we'd like to use vc->b but vc has been freed
+	 * (it got freed by start_adns_query->release_adns_continuation,
+	 *  noting that &vc->ac == vc)
 	 * so we have to use b.  This is why we plunked next_state
 	 * into b, not just vc->b.
 	 */
 	report_verify_failure(b, ugh);
-	p1st->st_suspended_md = NULL;
+	set_suspended(p1st,  NULL);
 	return STF_FAIL + INVALID_ID_INFORMATION;
     }
     else
@@ -1514,40 +1593,20 @@ quick_inI1_outR1_process_answer(struct verify_oppo_bundle *b
     return next_step;
 }
 
+/* forward definitions */
 static stf_status
-quick_inI1_outR1_cryptotail(struct qke_continuation *qke
+quick_inI1_outR1_cryptotail(struct dh_continuation *dh
 			    , struct pluto_crypto_req *r);
 
 static void
-quick_inI1_outR1_cryptocontinue(struct pluto_crypto_req_cont *pcrc
+quick_inI1_outR1_cryptocontinue2(struct pluto_crypto_req_cont *pcrc
 			      , struct pluto_crypto_req *r
-			      , err_t ugh)
-{
-    struct qke_continuation *qke = (struct qke_continuation *)pcrc;
-    struct state *const st = qke->st;
-    stf_status e;
+				 , err_t ugh);
 
-    DBG(DBG_CONTROLMORE
-	, DBG_log("quick inI1_outR1: calculated ke+nonce, sending R1"));
-
-    /* XXX should check out ugh */
-    passert(ugh == NULL);
-    passert(cur_state == NULL);
-    passert(st != NULL);
-
-    passert(st->st_connection != NULL);
-
-    set_cur_state(st);	/* we must reset before exit */
-    st->st_calculating=FALSE;
-    e = quick_inI1_outR1_cryptotail(qke, r);
-
-    if(qke->md != NULL) {
-	complete_state_transition(&qke->md, e);
-	release_md(qke->md);
-    }
-
-    reset_cur_state();
-}
+static void
+quick_inI1_outR1_cryptocontinue1(struct pluto_crypto_req_cont *pcrc
+				 , struct pluto_crypto_req *r
+				 , err_t ugh);
 
 static stf_status
 quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
@@ -1558,6 +1617,18 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
     struct connection *c = p1st->st_connection;
     ip_subnet *our_net = &b->my.net
 	, *his_net = &b->his.net;
+    struct hidden_variables hv;
+
+    {
+	char s1[SUBNETTOT_BUF],d1[SUBNETTOT_BUF];
+    
+	subnettot(our_net, 0, s1, sizeof(s1));
+	subnettot(his_net, 0, d1, sizeof(d1));
+	
+	openswan_log("the peer proposed: %s:%d/%d -> %s:%d/%d"
+		     , s1, c->spd.this.protocol, c->spd.this.port
+		     , d1, c->spd.that.protocol, c->spd.that.port);
+    }
 
     /* Now that we have identities of client subnets, we must look for
      * a suitable connection (our current one only matches for hosts).
@@ -1566,19 +1637,6 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 	struct connection *p = find_client_connection(c
 	    , our_net, his_net, b->my.proto, b->my.port, b->his.proto, b->his.port);
 
-#ifdef NAT_TRAVERSAL
-#ifdef I_KNOW_TRANSPORT_MODE_HAS_SECURITY_CONCERN_BUT_I_WANT_IT
-    if( (p1st->hidden_variables.st_nat_traversal & NAT_T_DETECTED)
-       && !(p1st->st_policy & POLICY_TUNNEL)
-       && (p1st->hidden_variables.st_nat_traversal & LELEM(NAT_TRAVERSAL_NAT_BHND_ME))
-       && (p == NULL) )
-        {
-          p = c;
-          DBG(DBG_CONTROL, DBG_log("using (something) old for transport mode connection \"%s\"", p->name));
-        }
-#endif
-#endif
-
 	if (p == NULL)
 	{
 	    /* This message occurs in very puzzling circumstances
@@ -1715,7 +1773,6 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 
 
 
-#ifdef VIRTUAL_IP
 	else if (is_virtual_connection(c))
 	{
 	    c->spd.that.client = *his_net;
@@ -1723,7 +1780,6 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 	    if (subnetishost(his_net) && addrinsubnet(&c->spd.that.host_addr, his_net))
 		c->spd.that.has_client = FALSE;
 	}
-#endif
     }
     passert((p1st->st_policy & POLICY_PFS)==0 || p1st->st_pfs_group != NULL );
 
@@ -1731,7 +1787,14 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
      * do any asynchronous cryptographic operations that we may need to
      * make it all work.
      */
-    
+
+    if ((p1st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) &&
+	(p1st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA)) {
+
+	hv = p1st->hidden_variables;
+	nat_traversal_natoa_lookup(md, &hv);
+    }
+
     /* now that we are sure of our connection, create our new state */
     {
 	struct state *const st = duplicate_state(p1st);
@@ -1769,6 +1832,9 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 
 	insert_state(st);	/* needs cookies, connection, and msgid */
 
+	/* copy hidden variables (possibly with changes) */
+	st->hidden_variables = hv;
+
 	/* copy the connection's
 	 * IPSEC policy into our state.  The ISAKMP policy is water under
 	 * the bridge, I think.  It will reflect the ISAKMP SA that we
@@ -1785,10 +1851,6 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 	else {
 	    st->hidden_variables.st_nat_traversal = 0;
 	}
-	if ((st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) &&
-	    (st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA)) {
-	    nat_traversal_natoa_lookup(md);
-	}
 #endif
 
 	passert(st->st_connection != NULL);
@@ -1825,7 +1887,7 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 
 	{
 	    struct qke_continuation *qke = alloc_thing(struct qke_continuation
-						      , "quick_inI1_outR1 KE");
+						      , "quick_outI1 KE");
 
 	    stf_status e;
 	    enum crypto_importance ci;
@@ -1836,7 +1898,7 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 	    qke->st = st;
 	    qke->isakmp_sa = p1st;
 	    qke->md = md;
-	    qke->qke_pcrc.pcrc_func = quick_inI1_outR1_cryptocontinue;
+	    qke->qke_pcrc.pcrc_func = quick_inI1_outR1_cryptocontinue1;
 
 	    if (st->st_pfs_group != NULL) {
 		e = build_ke(&qke->qke_pcrc, st, st->st_pfs_group, ci);
@@ -1851,11 +1913,114 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
     }
 }
 
+static void
+quick_inI1_outR1_cryptocontinue1(struct pluto_crypto_req_cont *pcrc
+				 , struct pluto_crypto_req *r
+				 , err_t ugh)
+{
+    struct qke_continuation *qke = (struct qke_continuation *)pcrc;
+    struct msg_digest *md = qke->md;
+    struct state *const st = qke->st;
+    stf_status e;
+
+    DBG(DBG_CONTROLMORE
+	, DBG_log("quick inI1_outR1: calculated ke+nonce, calculating DH"));
+
+    /* XXX should check out ugh */
+    passert(ugh == NULL);
+    passert(cur_state == NULL);
+    passert(st != NULL);
+
+    passert(st->st_connection != NULL);
+
+    set_cur_state(st);	/* we must reset before exit */
+    st->st_calculating=FALSE;
+    set_suspended(st, NULL);
+
+    /* we always calcualte a nonce */
+    unpack_nonce(&st->st_nr, r);
+
+    if (st->st_pfs_group != NULL) {
+	struct dh_continuation *dh = alloc_thing(struct dh_continuation
+						 , "quick outR1 DH");
+
+	unpack_KE(st, r, &st->st_gr);
+    
+	/* set up second calculation */
+	dh->md = md;
+	set_suspended(st, md);
+	dh->dh_pcrc.pcrc_func = quick_inI1_outR1_cryptocontinue2;
+	e = start_dh_secret(&dh->dh_pcrc, st
+			    , st->st_import
+			    , RESPONDER
+			    , st->st_pfs_group->group);
+	
+	if(e != STF_SUSPEND) {
+	    if(dh->md != NULL) {
+		complete_state_transition(&qke->md, e);
+		if(dh->md) release_md(qke->md);
+	    }
+	}
+	
+    } else {
+	/* but if PFS is off, we don't do a second DH, so
+	 * just call the continuation after making something up.
+	 */
+	struct dh_continuation dh;
+
+	dh.md=md;
+
+	e = quick_inI1_outR1_cryptotail(&dh, NULL);
+
+	if(dh.md != NULL) {
+	    /* note: use qke-> pointer */
+	    complete_state_transition(&qke->md, e);
+	    if(dh.md) release_md(qke->md);
+	}
+    }
+    reset_cur_state();
+	
+}
+
+static void
+quick_inI1_outR1_cryptocontinue2(struct pluto_crypto_req_cont *pcrc
+			      , struct pluto_crypto_req *r
+			      , err_t ugh)
+{
+    struct dh_continuation *dh = (struct dh_continuation *)pcrc;
+    struct msg_digest *md = dh->md;
+    struct state *const st = md->st;
+    stf_status e;
+
+    DBG(DBG_CONTROLMORE
+	, DBG_log("quick inI1_outR1: calculated DH, sending R1"));
+
+    /* XXX should check out ugh */
+    passert(ugh == NULL);
+    passert(cur_state == NULL);
+    passert(st != NULL);
+
+    passert(st->st_connection != NULL);
+
+    set_cur_state(st);	/* we must reset before exit */
+    st->st_calculating=FALSE;
+    set_suspended(st, NULL);
+
+    e = quick_inI1_outR1_cryptotail(dh, r);
+
+    if(dh->md != NULL) {
+	complete_state_transition(&dh->md, e);
+	if(dh->md) release_md(dh->md);
+    }
+
+    reset_cur_state();
+}
+
 static stf_status
-quick_inI1_outR1_cryptotail(struct qke_continuation *qke
+quick_inI1_outR1_cryptotail(struct dh_continuation *dh
 			   , struct pluto_crypto_req *r)
 {
-    struct msg_digest *md = qke->md;
+    struct msg_digest *md = dh->md;
     struct state *st = md->st;
     struct connection *c = st->st_connection;
     struct payload_digest *const id_pd = md->chain[ISAKMP_NEXT_ID];
@@ -1910,7 +2075,13 @@ quick_inI1_outR1_cryptotail(struct qke_continuation *qke
     
     {
 	int np;
+#ifdef IMPAIR_UNALIGNED_R1_MSG
+	char *padstr=getenv("PLUTO_UNALIGNED_R1_MSG");
 
+	if(padstr) {
+	    np = ISAKMP_NEXT_VID;
+	} else
+#endif
 	if(st->st_pfs_group != NULL) {
 	    np = ISAKMP_NEXT_KE;
 	} else if(id_pd != NULL) {
@@ -1920,24 +2091,48 @@ quick_inI1_outR1_cryptotail(struct qke_continuation *qke
 	}
 
 	/* Nr out */
-	if (!ship_nonce(&st->st_nr, r, &md->rbody
-			, np, "Nr"))
+	if (!justship_nonce(&st->st_nr, &md->rbody, np, "Nr"))
 	    return STF_INTERNAL_ERROR;
+
+
+#ifdef IMPAIR_UNALIGNED_R1_MSG
+	if(padstr) {
+	    pb_stream vid_pbs;
+	    int padsize;
+	    padsize = strtoul(padstr, NULL, 0);
+	    
+	    openswan_log("inserting fake VID payload of %u size", padsize);
+	    
+	    if(st->st_pfs_group != NULL) {
+		np = ISAKMP_NEXT_KE;
+	    } else if(id_pd != NULL) {
+		np = ISAKMP_NEXT_ID;
+	    } else {
+		np = ISAKMP_NEXT_NONE;
+	    }
+
+	    if (!out_generic(np,
+			     &isakmp_vendor_id_desc, &md->rbody, &vid_pbs))
+		return STF_INTERNAL_ERROR;
+	    
+	    if (!out_zero(padsize, &vid_pbs, "Filler VID"))
+		return STF_INTERNAL_ERROR;
+		
+	    close_output_pbs(&vid_pbs);
+	} 
+#endif
     }
     
-    /* [ KE ] out (for PFS) */
-    if (st->st_pfs_group != NULL) {
-	stf_status stat;
 
-	if (!ship_KE(st, r, &st->st_gr
-		     , &md->rbody
-		     , id_pd != NULL? ISAKMP_NEXT_ID : ISAKMP_NEXT_NONE))
+    /* [ KE ] out (for PFS) */
+    if (st->st_pfs_group != NULL && r!=NULL) {
+	if (!justship_KE(&st->st_gr
+			 , &md->rbody
+			 , id_pd != NULL? ISAKMP_NEXT_ID : ISAKMP_NEXT_NONE))
 	    return STF_INTERNAL_ERROR;
+
+	finish_dh_secret(st, r);
 	
-	stat = perform_dh_secret(st, RESPONDER, st->st_pfs_group->group);
-	if(stat != STF_OK) {
-	    return stat;
-	}
     }
     
     /* [ IDci, IDcr ] out */
@@ -1956,12 +2151,27 @@ quick_inI1_outR1_cryptotail(struct qke_continuation *qke
     }
 
 #ifdef NAT_TRAVERSAL
-    if ((st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) &&
-	(st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TRANSPORT) &&
-	(c->spd.that.has_client)) {
-	/** Remove client **/
-	addrtosubnet(&c->spd.that.host_addr, &c->spd.that.client);
-	c->spd.that.has_client = FALSE;
+#if 0
+    DBG_log("NAT-OA: %d tunnel: %d \n"
+		,(st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA)
+    		,(st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TRANSPORT));
+    if ((st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA) &&
+	(st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TRANSPORT)) {
+	/** Send NAT-OA if our address is NATed and if we use Transport Mode */
+	if (!nat_traversal_add_natoa(ISAKMP_NEXT_NONE, &md->rbody, md->st, FALSE)) {
+	    return STF_INTERNAL_ERROR;
+	}
+    }
+#endif
+#endif
+
+#ifdef TPM
+    {
+	pb_stream *pbs = &md->rbody;
+	size_t enc_len = pbs_offset(pbs) - sizeof(struct isakmp_hdr);
+
+	TCLCALLOUT_crypt("preHash", st,pbs,sizeof(struct isakmp_hdr),enc_len);
+	r_hashval = tpm_relocateHash(pbs);	
     }
 #endif
 
@@ -1998,11 +2208,18 @@ quick_inI1_outR1_cryptotail(struct qke_continuation *qke
  * (see RFC 2409 "IKE" 5.5)
  * Installs inbound and outbound IPsec SAs, routing, etc.
  */
+static stf_status
+quick_inR1_outI2_cryptotail(struct dh_continuation *dh
+			    , struct pluto_crypto_req *r);
+static void
+quick_inR1_outI2_continue(struct pluto_crypto_req_cont *pcrc
+			  , struct pluto_crypto_req *r
+			  , err_t ugh);
+
 stf_status
 quick_inR1_outI2(struct msg_digest *md)
 {
     struct state *const st = md->st;
-    const struct connection *c = st->st_connection;
 
     /* HASH(2) in */
     CHECK_QUICK_HASH(md
@@ -2024,39 +2241,134 @@ quick_inR1_outI2(struct msg_digest *md)
     /* [ KE ] in (for PFS) */
     RETURN_STF_FAILURE(accept_PFS_KE(md, &st->st_gr, "Gr", "Quick Mode R1"));
 
-    if (st->st_pfs_group != NULL) {
-	stf_status stat;
+    if(st->st_pfs_group) {
+	struct dh_continuation *dh = alloc_thing(struct dh_continuation
+						 , "quick outI2 DH");
 
-	stat = perform_dh_secret(st, INITIATOR, st->st_pfs_group->group);
-	if(stat != STF_OK) {
-	    return stat;
-	}
+	/* set up DH calculation */
+	dh->md = md;
+	set_suspended(st, md);
+	dh->dh_pcrc.pcrc_func = quick_inR1_outI2_continue;
+	return start_dh_secret(&dh->dh_pcrc, st
+			       , st->st_import
+			       , INITIATOR
+			       , st->st_pfs_group->group);
+    } else {
+	/* just call the tail function */
+	struct dh_continuation dh;
+
+	dh.md=md;
+	return quick_inR1_outI2_cryptotail(&dh, NULL);
     }
+}
+
+static void
+quick_inR1_outI2_continue(struct pluto_crypto_req_cont *pcrc
+			  , struct pluto_crypto_req *r
+			  , err_t ugh)
+{
+    struct dh_continuation *dh = (struct dh_continuation *)pcrc;
+    struct msg_digest *md = dh->md;
+    struct state *const st = md->st;
+    stf_status e;
+
+    DBG(DBG_CONTROLMORE
+	, DBG_log("quick inI1_outR1: calculated ke+nonce, calculating DH"));
+
+    /* XXX should check out ugh */
+    passert(ugh == NULL);
+    passert(cur_state == NULL);
+    passert(st != NULL);
+
+    passert(st->st_connection != NULL);
+
+    set_cur_state(st);	/* we must reset before exit */
+    st->st_calculating=FALSE;
+    set_suspended(st, NULL);
+
+    e = quick_inR1_outI2_cryptotail(dh, r);
+
+    if(dh->md != NULL) {
+	complete_state_transition(&dh->md, e);
+	if(dh->md) release_md(dh->md);
+    }
+    reset_cur_state();
+}
+
+stf_status
+quick_inR1_outI2_cryptotail(struct dh_continuation *dh
+			    , struct pluto_crypto_req *r)
+{
+    struct msg_digest *md = dh->md;
+    struct state *st = md->st;
+    struct connection *c = st->st_connection;
+
+    if (st->st_pfs_group != NULL && r!=NULL) {
+	finish_dh_secret(st, r);
+    }
+
+#ifdef NAT_TRAVERSAL
+    if ((st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) &&
+	(st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA)) {
+	nat_traversal_natoa_lookup(md, &st->hidden_variables);
+    }
+#endif
 
     /* [ IDci, IDcr ] in; these must match what we sent */
 
     {
-	struct payload_digest *const id_pd = md->chain[ISAKMP_NEXT_ID];
+	struct payload_digest *const IDci = md->chain[ISAKMP_NEXT_ID];
+	struct payload_digest *IDcr;
 
-	if (id_pd != NULL)
+	if (IDci != NULL)
 	{
 	    /* ??? we are assuming IPSEC_DOI */
 
 	    /* IDci (we are initiator) */
-
-	    if (!check_net_id(&id_pd->payload.ipsec_id, &id_pd->pbs
-	    , &st->st_myuserprotoid, &st->st_myuserport
-	    , &st->st_connection->spd.this.client
-	    , "our client"))
+	    if (!check_net_id(&IDci->payload.ipsec_id, &IDci->pbs
+			      , &st->st_myuserprotoid, &st->st_myuserport
+			      , &st->st_connection->spd.this.client
+			      , "our client"))
 		return STF_FAIL + INVALID_ID_INFORMATION;
 
+	    /* we checked elsewhere that we got two of them */
+	    IDcr = IDci->next;
+	    passert(IDcr != NULL);
+
 	    /* IDcr (responder is peer) */
 
-	    if (!check_net_id(&id_pd->next->payload.ipsec_id, &id_pd->next->pbs
-	    , &st->st_peeruserprotoid, &st->st_peeruserport
-	    , &st->st_connection->spd.that.client
-	    , "peer client"))
+	    if (!check_net_id(&IDcr->payload.ipsec_id, &IDcr->pbs
+			      , &st->st_peeruserprotoid, &st->st_peeruserport
+			      , &st->st_connection->spd.that.client
+			      , "peer client"))
 		return STF_FAIL + INVALID_ID_INFORMATION;
+
+	    /*
+	     * if there is a NATOA payload, then use it as
+	     *    &st->st_connection->spd.that.client, if the type
+	     * of the ID was FQDN
+	     */
+#ifdef NAT_TRAVERSAL
+	    if ((st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) &&
+		(st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA) &&
+		(IDcr->payload.ipsec_id.isaiid_idtype == ID_FQDN)) {
+		
+		char idfqdn[32], subnet_buf[SUBNETTOT_BUF];
+		int  idlen = pbs_room(&IDcr->pbs);
+		if(idlen > 31) idlen=31;
+		memcpy(idfqdn, IDcr->pbs.cur, idlen);
+		idfqdn[idlen]='\0';
+
+		addrtosubnet(&st->hidden_variables.st_nat_oa,
+			     &st->st_connection->spd.that.client);
+
+		subnettot(&st->st_connection->spd.that.client
+			  , 0, subnet_buf, sizeof(subnet_buf));
+		loglog(RC_LOG_SERIOUS, "IDcr was FQDN: %s, using NAT_OA=%s as IDcr"
+		       , idfqdn, subnet_buf);
+	    }
+#endif
+
 	}
 	else
 	{
@@ -2070,14 +2382,7 @@ quick_inR1_outI2(struct msg_digest *md)
 	    }
 	}
     }
-
-#ifdef NAT_TRAVERSAL
-	if ((st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) &&
-	    (st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA)) {
-	    nat_traversal_natoa_lookup(md);
-	}
-#endif
-
+    
     /* ??? We used to copy the accepted proposal into the state, but it was
      * never used.  From sa_pd->pbs.start, length pbs_room(&sa_pd->pbs).
      */
@@ -2086,17 +2391,54 @@ quick_inR1_outI2(struct msg_digest *md)
 
     /* HDR* out done */
 
-    /* HASH(3) out -- since this is the only content, no passes needed */
+    /* HASH(3) out -- sometimes, we add more content */
     {
 	u_char	/* set by START_HASH_PAYLOAD: */
 	    *r_hashval,	/* where in reply to jam hash value */
 	    *r_hash_start;	/* start of what is to be hashed */
 
+#ifdef IMPAIR_UNALIGNED_I2_MSG
+	{
+	    char *padstr=getenv("PLUTO_UNALIGNED_I2_MSG");
+	    
+	    if(padstr) {
+		pb_stream vid_pbs;
+		int padsize;
+		padsize = strtoul(padstr, NULL, 0);
+		
+		openswan_log("inserting fake VID payload of %u size", padsize);
+		START_HASH_PAYLOAD(md->rbody, ISAKMP_NEXT_VID);
+		
+		if (!out_generic(ISAKMP_NEXT_NONE,
+				 &isakmp_vendor_id_desc, &md->rbody, &vid_pbs))
+		    return STF_INTERNAL_ERROR;
+		
+		if (!out_zero(padsize, &vid_pbs, "Filler VID"))
+		    return STF_INTERNAL_ERROR;
+		
+		close_output_pbs(&vid_pbs);
+	    } else {
+		START_HASH_PAYLOAD(md->rbody, ISAKMP_NEXT_NONE);
+	    }
+	}
+#else
 	START_HASH_PAYLOAD(md->rbody, ISAKMP_NEXT_NONE);
+#endif
+
+#ifdef TPM
+	{
+	    pb_stream *pbs = &md->rbody;
+	    size_t enc_len = pbs_offset(pbs) - sizeof(struct isakmp_hdr);
+	    
+	    TCLCALLOUT_crypt("preHash", st,pbs,sizeof(struct isakmp_hdr),enc_len);
+	    r_hashval = tpm_relocateHash(pbs);	
+	}
+#endif
+
 	(void)quick_mode_hash3(r_hashval, st);
     }
 
-    /* Derive new keying material */
+   /* Derive new keying material */
     compute_keymats(st);
 
     /* Tell the kernel to establish the inbound, outbound, and routing part
