@@ -13,7 +13,10 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: plutomain.c,v 1.102.2.7 2007-04-06 17:10:37 paul Exp $
+ * Modifications to use OCF interface written by
+ * Daniel Djamaludin <danield@cyberguard.com>
+ * Copyright (C) 2004-2005 Intel Corporation.  All Rights Reserved.
+ *
  */
 
 #include <stdio.h>
@@ -27,21 +30,20 @@
 #include <sys/un.h>
 #include <fcntl.h>
 #include <getopt.h>
-#include <resolv.h>
 #include <arpa/nameser.h>	/* missing from <resolv.h> on old systems */
-#include <sys/queue.h>
 
 #include <openswan.h>
 
-#include <pfkeyv2.h>
-#include <pfkey.h>
+#include <openswan/pfkeyv2.h>
+#include <openswan/pfkey.h>
 
+#include "sysdep.h"
 #include "constants.h"
+#include "oswconf.h"
 #include "defs.h"
 #include "id.h"
 #include "x509.h"
 #include "pgp.h"
-#include "paths.h"
 #include "certs.h"
 #include "ac.h"
 #include "smartcard.h"
@@ -71,19 +73,25 @@
 #include "vendor.h"
 #include "pluto_crypt.h"
 
-#ifdef VIRTUAL_IP
 #include "virtual.h"
-#endif
 
 #ifdef NAT_TRAVERSAL
 #include "nat_traversal.h"
 #endif
 
+#ifdef TPM
+#include <tcl.h>
+#include "tpm/tpm.h"
+#endif
+
+#ifdef HAVE_OCF
+#include "ocf_pk.h"
+#endif
+
 #ifndef IPSECDIR
 #define IPSECDIR "/etc/ipsec.d"
 #endif
 
-const char *ipsec_dir = IPSECDIR;
 const char *ctlbase = "/var/run/pluto";
 
 openswan_passert_fail_t openswan_passert_fail = passert_fail;
@@ -115,7 +123,7 @@ usage(const char *mess)
 	    " [--use-netkey]"
 	    " [--use-nostack]"         /* old --no_klips */
 	    " \\\n\t"
-	    "[--interface <ifname>]"
+	    "[--interface <ifname|ifaddr>]"
 	    " [--ikeport <port-number>]"
 	    " \\\n\t"
 	    "[--ctlbase <path>]"
@@ -142,8 +150,10 @@ usage(const char *mess)
 	    "[--debug-control]"
 	    "[--debug-lifecycle]"
 	    " [--debug-klips]"
+	    " [--debug-x509]"
 	    " [--debug-dns]"
 	    " [--debug-oppo]"
+	    " [--debug-oppoinfo]"
 	    " [--debug-dpd]"
 	    " [ --debug-private]"
 	    " [ --debug-pfkey]"
@@ -155,10 +165,8 @@ usage(const char *mess)
 	    " \\\n\t"
             "[--force_keepalive] [--disable_port_floating]"
 #endif
-#ifdef VIRTUAL_IP
 	   " \\\n\t"
 	   "[--virtual_private <network_list>]"
-#endif
 	    "\n"
 	"Openswan %s\n"
 	, ipsec_version_code());
@@ -257,16 +265,17 @@ enum kernel_interface kern_interface = AUTO_PICK;
 
 char **global_argv;
 int    global_argc;
+bool log_to_stderr_desired = FALSE;
 
 int
 main(int argc, char **argv)
 {
     bool fork_desired = TRUE;
-    bool log_to_stderr_desired = FALSE;
     int lockfd;
     char* ocspuri = NULL;
     int nhelpers = -1;
     char *coredir;
+    const struct osw_conf_options *oco;
 
 #ifdef NAT_TRAVERSAL
     /** Overridden by nat_traversal= in ipsec.conf */
@@ -275,10 +284,8 @@ main(int argc, char **argv)
     unsigned int keep_alive = 0;
     bool force_keepalive = FALSE;
 #endif
-#ifdef VIRTUAL_IP
     /** Overridden by virtual_private= in ipsec.conf */
     char *virtual_private = NULL;
-#endif
 
     global_argv = argv;
     global_argc = argc;
@@ -287,6 +294,10 @@ main(int argc, char **argv)
     /* see if there is an environment variable */
     coredir = getenv("PLUTO_CORE_DIR");
 
+    if(getenv("PLUTO_WAIT_FOR_GDB")) {
+	sleep(120);
+    }
+
     /* handle arguments */
     for (;;)
     {
@@ -300,6 +311,7 @@ main(int argc, char **argv)
 	    { "stderrlog", no_argument, NULL, 'e' },
 	    { "noklips", no_argument, NULL, 'n' },
 	    { "use-nostack",  no_argument, NULL, 'n' },
+	    { "use-none",     no_argument, NULL, 'n' },
 	    { "nocrsend", no_argument, NULL, 'c' },
 	    { "strictcrlpolicy", no_argument, NULL, 'r' },
 	    { "crlcheckinterval", required_argument, NULL, 'x'},
@@ -311,6 +323,7 @@ main(int argc, char **argv)
 	    { "use-auto",  no_argument, NULL, 'G' },
 	    { "usenetkey", no_argument, NULL, 'K' },
 	    { "use-netkey", no_argument, NULL, 'K' },
+	    { "use-mast",   no_argument, NULL, 'M' },
 	    { "interface", required_argument, NULL, 'i' },
 	    { "ikeport", required_argument, NULL, 'p' },
 	    { "ctlbase", required_argument, NULL, 'b' },
@@ -336,9 +349,7 @@ main(int argc, char **argv)
 	    { "debug-nattraversal", no_argument, NULL, '5' },
 	    { "debug-nat-t", no_argument, NULL, '5' },
 #endif
-#ifdef VIRTUAL_IP
 	    { "virtual_private", required_argument, NULL, '6' },
-#endif
 	    { "nhelpers", required_argument, NULL, 'j' },
 #ifdef DEBUG
 	    { "debug-none", no_argument, NULL, 'N' },
@@ -353,8 +364,10 @@ main(int argc, char **argv)
 	    { "debug-klips", no_argument, NULL, DBG_KLIPS + DBG_OFFSET },
 	    { "debug-dns", no_argument, NULL, DBG_DNS + DBG_OFFSET },
 	    { "debug-oppo", no_argument, NULL, DBG_OPPO + DBG_OFFSET },
+	    { "debug-oppoinfo", no_argument, NULL, DBG_OPPOINFO + DBG_OFFSET },
 	    { "debug-controlmore", no_argument, NULL, DBG_CONTROLMORE + DBG_OFFSET },
 	    { "debug-dpd", no_argument, NULL, DBG_DPD + DBG_OFFSET },
+            { "debug-x509", no_argument, NULL, DBG_X509 + DBG_OFFSET },
 	    { "debug-private", no_argument, NULL, DBG_PRIVATE + DBG_OFFSET },
 	    { "debug-pfkey", no_argument, NULL, DBG_PFKEY + DBG_OFFSET },
 
@@ -421,7 +434,7 @@ main(int argc, char **argv)
 
                 if (*endptr != '\0' || endptr == optarg
 		    || count < -1)
-                    usage("<nhelpers> must be a positive number, 0 or -1");
+                    usage("<interval-time> must be a positive number, 0 or -1");
                 nhelpers = count;
             }
 	    continue;
@@ -442,6 +455,10 @@ main(int argc, char **argv)
 	    kern_interface = USE_KLIPS;
 	    continue;
 
+	case 'M':       /* --use-mast */
+	    kern_interface = USE_MASTKLIPS;
+	    continue;
+
 	case 'K':       /* --use-netkey */
 	    kern_interface = USE_NETKEY;
 	    continue;
@@ -488,7 +505,7 @@ main(int argc, char **argv)
 	    uniqueIDs = TRUE;
 	    continue;
 
-	case 'i':	/* --interface <ifname> */
+	case 'i':	/* --interface <ifname|ifaddr> */
 	    if (!use_interface(optarg))
 		usage("too many --interface specifications");
 	    continue;
@@ -522,11 +539,11 @@ main(int argc, char **argv)
 	    continue;
 
 	case 's':	/* --secretsfile <secrets-file> */
-	    shared_secrets_file = optarg;
+	    pluto_shared_secrets_file = optarg;
 	    continue;
 
 	case 'f':	/* --ipsecdir <ipsec-dir> */
-	    ipsec_dir = optarg;
+	    (void)osw_init_ipsecdir(optarg);
 	    continue;
 
 	case 'a':	/* --adns <pathname> */
@@ -568,11 +585,9 @@ main(int argc, char **argv)
 	    base_debugging |= DBG_NATT;
 	    continue;
 #endif
-#ifdef VIRTUAL_IP
 	case '6':	/* --virtual_private */
 	    virtual_private = optarg;
 	    continue;
-#endif
 
 	default:
 #ifdef DEBUG
@@ -596,6 +611,7 @@ main(int argc, char **argv)
 	chdir(coredir);
     }
 
+    oco = osw_init_options();
     lockfd = create_lock();
 
     /* select between logging methods */
@@ -605,13 +621,12 @@ main(int argc, char **argv)
     else
 	log_to_stderr = FALSE;
 
-    /* set the logging function of pfkey debugging */
 #ifdef DEBUG
-    pfkey_debug_func = DBG_log;
-    pfkey_error_func = DBG_log;
-#else
-    pfkey_debug_func = NULL;
-    pfkey_error_func = NULL;
+#if 0
+    if(kernel_ops->set_debug) {
+	(*kernel_ops->set_debug)(cur_debugging, DBG_log, DBG_log);
+    }
+#endif
 #endif
 
     /** create control socket.
@@ -720,24 +735,30 @@ main(int argc, char **argv)
         const char *v = init_pluto_vendorid();
 	const char *vc = ipsec_version_code();
 
-        openswan_log("Starting Pluto (Openswan Version %s%s; Vendor ID %s)"
-            , vc
-            , compile_time_interop_options
-            , v);
-	if(vc[0]=='c' && vc[1]=='v' && vc[2]=='s') {
+        openswan_log("Starting Pluto (Openswan Version %s%s; Vendor ID %s) pid:%u"
+		     , vc
+		     , compile_time_interop_options
+		     , v, getpid());
+#else
+        openswan_log("Starting Pluto (Openswan Version %s%s) pid:%u"
+		     , ipsec_version_code()
+		     , compile_time_interop_options, getpid());
+#endif
+
+	if((vc[0]=='c' && vc[1]=='v' && vc[2]=='s') ||
+	   (vc[2]=='g' && vc[3]=='i' && vc[4]=='t')) {
 	    /*
-	     * when people build RPMs from CVS, make sure they get blamed
-	     * appropriately, and that we get some way to identify who
-	     * did it, and when they did it. Use string concat, so that
-	     * strings the binary can or classic SCCS "what", will find
+	     * when people build RPMs from CVS or GIT, make sure they
+	     * get blamed appropriately, and that we get some way to
+	     * identify who did it, and when they did it. Use string concat,
+	     * so that strings the binary can or classic SCCS "what", will find
 	     * stuff too.
 	     */
-	    openswan_log("@(#) built on "__DATE__":"__TIME__":"BUILDER);
+	    openswan_log("@(#) built on "__DATE__":" __TIME__ " by " BUILDER);
 	}
-#else
-        openswan_log("Starting Pluto (Openswan Version %s%s)"
-            , ipsec_version_code()
-            , compile_time_interop_options);
+
+#if defined(USE_1DES)
+	openswan_log("WARNING: 1DES is enabled");
 #endif
     }
 
@@ -751,20 +772,25 @@ main(int argc, char **argv)
     init_nat_traversal(nat_traversal, keep_alive, force_keepalive, nat_t_spf);
 #endif
 
-#ifdef VIRTUAL_IP
     init_virtual_ip(virtual_private);
-#endif
     init_rnd_pool();
     init_secret();
     init_states();
     init_connections();
     init_crypto();
     init_crypto_helpers(nhelpers);
+#ifdef HAVE_OCF
+    load_cryptodev();
+#endif
     init_demux();
     init_kernel();
     init_adns();
     init_id();
 
+#ifdef TPM
+    init_tpm();
+#endif
+
 #ifdef HAVE_THREADS
     init_fetch();
 #endif
@@ -772,11 +798,11 @@ main(int argc, char **argv)
     ocsp_set_default_uri(ocspuri);
 
     /* loading X.509 CA certificates */
-    load_authcerts("CA cert", CA_CERT_PATH, AUTH_CA);
+    load_authcerts("CA cert", oco->cacerts_dir, AUTH_CA);
     /* loading X.509 AA certificates */
-    load_authcerts("AA cert", AA_CERT_PATH, AUTH_AA);
+    load_authcerts("AA cert", oco->aacerts_dir, AUTH_AA);
     /* loading X.509 OCSP certificates */
-    load_authcerts("OCSP cert", OCSP_CERT_PATH, AUTH_OCSP);
+    load_authcerts("OCSP cert", oco->ocspcerts_dir, AUTH_OCSP);
 
     /* loading X.509 CRLs */
     load_crls();
@@ -807,6 +833,10 @@ exit_pluto(int status)
     /* free memory allocated by initialization routines.  Please don't
        forget to do this. */
 
+#ifdef TPM
+    free_tpm();
+#endif
+
 #ifdef HAVE_THREADS
     free_crl_fetch();          /* free chain of crl fetch requests */
 #endif
