@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: spdb_struct.c,v 1.13.2.17 2008-04-24 18:54:00 paul Exp $
+ * RCSID $Id: spdb_struct.c,v 1.19 2005/09/26 23:35:28 mcr Exp $
  */
 
 #include <stdio.h>
@@ -20,12 +20,12 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <sys/queue.h>
 
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
-#include "pfkeyv2.h"
+#include "openswan/pfkeyv2.h"
 
+#include "sysdep.h"
 #include "constants.h"
 #include "oswlog.h"
 
@@ -47,6 +47,7 @@
 #include "log.h"
 #include "spdb.h"
 #include "whack.h"	/* for RC_LOG_SERIOUS */
+#include "plutoalg.h"
 
 #include "sha1.h"
 #include "md5.h"
@@ -131,56 +132,26 @@ out_sa(pb_stream *outs
 
 
     if(oakley_mode) {
+	const char *modestr;
+	const char *alginfo;
+
 	revised_sadb=oakley_alg_makedb(st->st_connection->alg_info_ike
 				       , sadb
 				       , aggressive_mode ? 1 : -1);
-    } else {
-	revised_sadb=kernel_alg_makedb(st->st_connection->alg_info_esp, TRUE);
+	modestr = "ike";
+	alginfo = st->st_connection->alg_ike;
 
-	/* add IPcomp proposal if policy asks for it */
-
-	if (revised_sadb && ((st->st_policy) & POLICY_COMPRESS)) {
-
-	    struct db_trans *ipcomp_trans = malloc (sizeof (struct db_trans));
-
-	    /* allocate space for 2 proposals */
-	    struct db_prop *ipcomp_prop = malloc (sizeof (struct db_prop) * 2);
-
-	    if (ipcomp_trans && ipcomp_prop) {
-
-		passert (revised_sadb->prop_conjs->prop_cnt == 1);
-
-		/* construct the IPcomp proposal  */
-		ipcomp_trans->transid = IPCOMP_DEFLATE;
-		ipcomp_trans->attrs = NULL;
-		ipcomp_trans->attr_cnt = 0;
-
-		/* copy the original proposal */
-		ipcomp_prop[0].protoid	 = revised_sadb->prop_conjs->props->protoid;
-		ipcomp_prop[0].trans	 = revised_sadb->prop_conjs->props->trans;
-		ipcomp_prop[0].trans_cnt = revised_sadb->prop_conjs->props->trans_cnt;
-
-		/* and add our IPcomp proposal */
-		ipcomp_prop[1].protoid = PROTO_IPCOMP;
-		ipcomp_prop[1].trans = ipcomp_trans;
-		ipcomp_prop[1].trans_cnt = 1;
-
-		/* free the old proposal, and ... */
-		pfree (revised_sadb->prop_conjs->props);
-
-		/* ... use our new one instead */
-		revised_sadb->prop_conjs->props = ipcomp_prop;
-		revised_sadb->prop_conjs->prop_cnt += 1;
-
-	    }
-	    else {
-		/* couldn't malloc something, so skip adding the proposal */
-		if (ipcomp_trans)
-		    free (ipcomp_trans);
-		if (ipcomp_prop)
-		    free (ipcomp_prop);
-	    }
+	/* this is really checked upon load, but we double check here. */
+	if(revised_sadb == NULL && alginfo!=NULL) {
+	    loglog(RC_NOALGO, "%s algorithm string: \"%s\" results in no permitted algorithms"
+		   , modestr, alginfo);
+	    return FALSE;
 	}
+    } else {
+	revised_sadb=kernel_alg_makedb(st->st_connection->policy
+				       , st->st_connection->alg_info_esp
+				       , TRUE);
+
     }
 
     /* more sanity */
@@ -271,8 +242,8 @@ out_sa(pb_stream *outs
 		: IPSEC_DOI_SPI_SIZE;
 
 	    DBG(DBG_EMITTING, 
-		DBG_log("out_sa pcn: %d pn: %d<%d valid_count: %d",
-			pcn, pn, pc->prop_cnt, valid_prop_cnt));
+		DBG_log("out_sa pcn: %d pn: %d<%d valid_count: %d trans_cnt: %d",
+			pcn, pn, pc->prop_cnt, valid_prop_cnt, p->trans_cnt));
 
 	    /* but, skip things if the transform count is zero */
 	    if(p->trans_cnt == 0) continue; 
@@ -306,11 +277,7 @@ out_sa(pb_stream *outs
 	    {
 		ipsec_spi_t *spi_ptr = NULL;
 		int proto = 0;
-		bool *spi_generated = FALSE;
-
-		spi_generated = NULL;
-
-		spi_generated = NULL;
+		bool *spi_generated;
 
 		spi_generated = NULL;
 
@@ -554,6 +521,80 @@ decode_long_duration(pb_stream *pbs)
     return val;
 }
 
+/* Preparse the body of an ISAKMP SA Payload and find which policy is required
+ * to match the packet. Errors are just ignored and will be detected and
+ * handled later in parse_isakmp_sa_body().
+ *
+ * All we want for the moment is to know whether peer is using RSA or PSK.
+ */
+lset_t preparse_isakmp_sa_body(pb_stream *sa_pbs)
+{
+    pb_stream proposal_pbs;
+    struct isakmp_proposal proposal;
+    pb_stream trans_pbs;
+    struct isakmp_transform trans;
+    u_char *attr_start;
+    size_t attr_len;
+    struct isakmp_attribute a;
+    pb_stream attr_pbs;
+    u_int32_t ipsecdoisit;
+    unsigned trans_left;
+    lset_t policy = 0;
+
+    if (!in_struct(&ipsecdoisit, &ipsec_sit_desc, sa_pbs, NULL))
+	return LEMPTY;
+
+    if (!in_struct(&proposal, &isakmp_proposal_desc, sa_pbs, &proposal_pbs))
+	return LEMPTY;
+
+    if (proposal.isap_spisize > MAX_ISAKMP_SPI_SIZE)
+	return LEMPTY;
+
+    if (proposal.isap_spisize > 0)
+    {
+	u_char junk_spi[MAX_ISAKMP_SPI_SIZE];
+
+	if (!in_raw(junk_spi, proposal.isap_spisize, &proposal_pbs, "Oakley SPI"))
+    return LEMPTY;
+    }
+
+    trans_left = proposal.isap_notrans;
+    while (trans_left--) {
+	if (!in_struct(&trans, &isakmp_isakmp_transform_desc, &proposal_pbs,
+	    &trans_pbs))
+	    return LEMPTY;
+
+	attr_start = trans_pbs.cur;
+	attr_len = pbs_left(&trans_pbs);
+
+	while (pbs_left(&trans_pbs) != 0) {
+	    if (!in_struct(&a, &isakmp_oakley_attribute_desc, &trans_pbs,
+		&attr_pbs))
+		return LEMPTY;
+	    switch (a.isaat_af_type) {
+		case OAKLEY_AUTHENTICATION_METHOD | ISAKMP_ATTR_AF_TV:
+		    switch (a.isaat_lv) {
+			case XAUTHInitPreShared:
+			case OAKLEY_PRESHARED_KEY:
+				policy |= POLICY_PSK;
+			    break;
+			case XAUTHInitRSA:
+			case OAKLEY_RSA_SIG:
+				policy |= POLICY_RSASIG;
+			    break;
+		    }
+		    break;
+	    }
+	}
+    }
+
+    if ((policy & POLICY_PSK) && (policy & POLICY_RSASIG))
+	policy &= ~(POLICY_PSK|POLICY_RSASIG);
+
+    return policy;
+}
+
+
 /**
  * Parse the body of an ISAKMP SA Payload (i.e. Phase 1 / Main Mode).
  * Various shortcuts are taken.  In particular, the policy, such as
@@ -812,6 +853,8 @@ parse_isakmp_sa_body(
 		    {
 #if defined(USE_1DES)
 		    case OAKLEY_DES_CBC:
+			openswan_log("1DES is not encryption");
+			/* FALL THROUGH */
 #endif
 		    case OAKLEY_3DES_CBC:
 			ta.encrypt = val;
@@ -901,18 +944,18 @@ parse_isakmp_sa_body(
 			else
 			{
 			    /* check that we can find a preshared secret */
-			    struct connection *c = st->st_connection;
+			    struct connection *con = st->st_connection;
 
-			    if (get_preshared_secret(c) == NULL)
+			    if (get_preshared_secret(con) == NULL)
 			    {
 				char mid[IDTOA_BUF]
 				    , hid[IDTOA_BUF];
 
-				idtoa(&c->spd.this.id, mid, sizeof(mid));
-				if (his_id_was_instantiated(c))
+				idtoa(&con->spd.this.id, mid, sizeof(mid));
+				if (his_id_was_instantiated(con))
 				    strcpy(hid, "%any");
 				else
-				    idtoa(&c->spd.that.id, hid, sizeof(hid));
+				    idtoa(&con->spd.that.id, hid, sizeof(hid));
 				ugh = builddiag("Can't authenticate: no preshared key found for `%s' and `%s'"
 				    , mid, hid);
 			    }
@@ -986,7 +1029,7 @@ parse_isakmp_sa_body(
 		    ta.group = lookup_group(val);
 		    if (ta.group == NULL)
 		    {
-			ugh = builddiag("Diffie-Hellman group %d is not a supported modp group" , val);
+			ugh = "only OAKLEY_GROUP_MODP1024 and OAKLEY_GROUP_MODP1536 supported";
 			break;
 		    }
 		    break;
@@ -1058,16 +1101,10 @@ parse_isakmp_sa_body(
 		     * check if this keylen is compatible with 
 		     * specified alg_info_ike
 		     */
-                   if(ta.encrypt == OAKLEY_3DES_CBC) {
-                       ta.enckeylen = 24;
-                   } else if(ta.encrypt == OAKLEY_DES_CBC) {
-                       ta.enckeylen = 8;
-                   } else {
-                       if (!ike_alg_enc_ok(ta.encrypt, val, c->alg_info_ike, &ugh)) {
-                           ugh = "peer proposed key_len not valid for encrypt algo setup specified";
-                       }
-                       ta.enckeylen=val;
+		    if (!ike_alg_enc_ok(ta.encrypt, val, c->alg_info_ike, &ugh)) {
+			ugh = "peer proposed key_len not valid for encrypt algo setup specified";
 		    }
+		    ta.enckeylen=val;
 		    break;
 #else
 		case OAKLEY_KEY_LENGTH | ISAKMP_ATTR_AF_TV:
@@ -1269,7 +1306,7 @@ init_am_st_oakley(struct state *st, lset_t policy)
     passert(prop->trans_cnt == 1);
     trans = &prop->trans[0];
 
-    passert(trans->attr_cnt == 4 || trans->attr_cnt == 5);
+    passert(trans->attr_cnt == 4 || trans->attr_cnt == 5); 
     enc  = &trans->attrs[0];
     hash = &trans->attrs[1];
     auth = &trans->attrs[2];
@@ -1809,6 +1846,10 @@ parse_ipsec_sa_body(
 	esp_spi = 0;
 	ah_spi = 0;
 
+	memset(&ah_proposal, 0, sizeof(ah_proposal));
+	memset(&esp_proposal, 0, sizeof(esp_proposal));
+	memset(&ipcomp_proposal, 0, sizeof(ipcomp_proposal));
+
 	/* for each proposal in the conjunction */
 	do {
 
@@ -2089,8 +2130,9 @@ parse_ipsec_sa_body(
 
 #ifdef KERNEL_ALG
 		if(c->alg_info_esp) {
-		    ugh = kernel_alg_esp_enc_ok(esp_attrs.transid, esp_attrs.key_len,
-						c->alg_info_esp);
+		    ugh = kernel_alg_esp_enc_ok(esp_attrs.transid
+						, esp_attrs.key_len
+						, c->alg_info_esp);
 		}
 #endif
 
@@ -2169,22 +2211,22 @@ parse_ipsec_sa_body(
 		    loglog(RC_LOG_SERIOUS
 			   , "AH and ESP transforms disagree about encapsulation; TUNNEL presumed");
 		}
-#ifdef KERNEL_ALG
-		/* 
-		 * ML: at last check for allowed transforms in alg_info_esp 
-		 *     (ALG_INFO_F_STRICT flag)
-		 *
-		 */
-		if (c->alg_info_esp!=NULL
-		    && !kernel_alg_esp_ok_final(esp_attrs.transid, esp_attrs.key_len,
-						esp_attrs.auth, c->alg_info_esp))
-			continue;
-#endif
 
 		break;	/* we seem to be happy */
 	    }
 	    if (tn == esp_proposal.isap_notrans)
 		continue;	/* we didn't find a nice one */
+#ifdef KERNEL_ALG
+	    /* 
+	     * ML: at last check for allowed transforms in alg_info_esp 
+	     *     (ALG_INFO_F_STRICT flag)
+	     *
+	     */
+	    if (c->alg_info_esp!=NULL
+		&& !kernel_alg_esp_ok_final(esp_attrs.transid, esp_attrs.key_len,
+					    esp_attrs.auth, c->alg_info_esp))
+		    continue;
+#endif
 	    esp_attrs.spi = esp_spi;
 	    inner_proto = IPPROTO_ESP;
 	    if (esp_attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL)
@@ -2362,6 +2404,225 @@ parse_ipsec_sa_body(
     return NO_PROPOSAL_CHOSEN;
 }
 
+#define AD(x) x, elemsof(x)	/* Array Description */
+#define AD_NULL NULL, 0
+/*
+ * empty structure, for clone use.
+ */
+static struct db_attr otempty[] = {
+	{ OAKLEY_ENCRYPTION_ALGORITHM, -1 },
+	{ OAKLEY_HASH_ALGORITHM,       -1 },
+	{ OAKLEY_AUTHENTICATION_METHOD, -1 },
+	{ OAKLEY_GROUP_DESCRIPTION,    -1 },
+	{ OAKLEY_KEY_LENGTH,    -1 },
+	};
+
+static struct db_trans oakley_trans_empty[] = {
+	{ KEY_IKE, AD(otempty) },
+    };
+
+static struct db_prop oakley_pc_empty[] =
+    { { PROTO_ISAKMP, AD(oakley_trans_empty) } };
+
+static struct db_prop_conj oakley_props_empty[] = { { AD(oakley_pc_empty) } };
+
+struct db_sa oakley_empty = { AD(oakley_props_empty) };
+
+/*
+ * 	Create an OAKLEY proposal based on alg_info and policy
+ */
+struct db_sa *
+oakley_alg_makedb(struct alg_info_ike *ai
+		  , struct db_sa *base
+		  , int maxtrans)
+{
+    /* struct db_context inprog UNUSED; */
+    struct db_sa *gsp = NULL;
+    struct db_sa *emp_sp = NULL;
+    struct ike_info *ike_info;
+    unsigned ealg, halg, modp, eklen=0;
+    struct encrypt_desc *enc_desc;
+    int transcnt = 0;
+    int i;
+
+    /*
+     * start by copying the proposal that would have been picked by
+     * standard defaults.
+     */
+
+    if (!ai) {
+	DBG(DBG_CRYPT,DBG_log("no IKE algorithms for this connection "));
+	
+	goto fail;
+    }
+
+    gsp = NULL;
+
+    /*
+     * for each group, we will create a new proposal item, and then
+     * append it to the list of transforms in the conjoint point.
+     *
+     * when creating each item, we will use the first transform
+     * from the base item as the template.
+     */
+    ALG_INFO_IKE_FOREACH(ai, ike_info, i) {
+
+	if(ike_info->ike_default == FALSE) {
+	    struct db_attr  *enc, *hash, *auth, *grp, *enc_keylen, *new_auth;
+	    struct db_trans *trans;
+	    struct db_prop  *prop;
+	    struct db_prop_conj *cprop;
+	    
+	    ealg = ike_info->ike_ealg;
+	    halg = ike_info->ike_halg;
+	    modp = ike_info->ike_modp;
+	    eklen= ike_info->ike_eklen;
+	    
+	    if (!ike_alg_enc_present(ealg)) {
+		DBG_log("oakley_alg_makedb() "
+			"ike enc ealg=%d not present",
+			ealg);
+		continue;
+	    }
+	    if (!ike_alg_hash_present(halg)) {
+		DBG_log("oakley_alg_makedb() "
+			"ike hash halg=%d not present",
+			halg);
+		continue;
+	    }
+	    enc_desc = ike_alg_get_encrypter(ealg);
+	    
+	    passert(enc_desc != NULL);
+	    if (eklen 
+		&& (eklen < enc_desc->keyminlen
+		    || eklen >  enc_desc->keymaxlen))
+		
+		{
+		    DBG_log("ike_alg_db_new() "
+			    "ealg=%d (specified) keylen:%d, "
+			    "not valid "
+			    "min=%d, max=%d"
+			    , ealg
+			    , eklen
+			    , enc_desc->keyminlen
+			    , enc_desc->keymaxlen
+			    );
+		    continue;
+		}
+	    
+	    /* okay copy the basic item, and modify it. */
+	    if(eklen > 0)
+	    {
+		emp_sp = sa_copy_sa(&oakley_empty, 0);
+		cprop = &base->prop_conjs[0];
+		prop = &cprop->props[0];
+		trans = &prop->trans[0];
+		new_auth = &trans->attrs[2];
+
+		cprop = &emp_sp->prop_conjs[0];
+		prop = &cprop->props[0];
+		trans = &prop->trans[0];
+		auth = &trans->attrs[2];
+		*auth = *new_auth;
+	    }
+	    else
+		emp_sp = sa_copy_sa_first(base);
+
+	    passert(emp_sp->prop_conj_cnt == 1);
+	    cprop = &emp_sp->prop_conjs[0];
+	    
+	    passert(cprop->prop_cnt == 1);
+	    prop = &cprop->props[0];
+	    
+	    passert(prop->trans_cnt == 1);
+	    trans = &prop->trans[0];
+	    
+	    passert(trans->attr_cnt == 4 || trans->attr_cnt == 5);
+	    enc  = &trans->attrs[0];
+	    hash = &trans->attrs[1];
+	    auth = &trans->attrs[2];
+	    grp  = &trans->attrs[3];
+
+	    if(eklen > 0) {
+		enc_keylen = &trans->attrs[4];
+		enc_keylen->val = eklen;
+	    }
+
+	    passert(enc->type == OAKLEY_ENCRYPTION_ALGORITHM);
+	    if(ealg > 0) {
+		enc->val = ealg;
+	    }
+	    
+	    modp = ike_info->ike_modp;
+	    eklen= ike_info->ike_eklen;
+	    
+	    passert(hash->type == OAKLEY_HASH_ALGORITHM);
+	    if(halg > 0) {
+		hash->val = halg;
+	    }
+	    
+	    passert(auth->type == OAKLEY_AUTHENTICATION_METHOD);
+	    /* no setting for auth type for IKE */
+	    
+	    passert(grp->type == OAKLEY_GROUP_DESCRIPTION);
+	    if(modp > 0) {
+		grp->val = modp;
+	    }
+	} else {
+	    emp_sp = sa_copy_sa(base, 0);
+	}
+
+	if(maxtrans == 1) {
+	    if(transcnt == 0) {
+		DBG(DBG_CONTROL, DBG_log("using transform (%d,%d,%d,%ld)"
+					 , ike_info->ike_ealg
+					 , ike_info->ike_halg
+					 , ike_info->ike_modp
+					 , (long)ike_info->ike_eklen));
+		if(gsp) {
+		    free_sa(gsp);
+		}
+		gsp = emp_sp;
+	    } else {
+		free_sa(emp_sp);
+	    }
+
+	    if(transcnt > 0) {
+		if(transcnt == 1) {
+		    loglog(RC_LOG_SERIOUS
+			   
+			   , "multiple transforms were set in aggressive mode. Only first one used.");
+		}
+
+		loglog(RC_LOG_SERIOUS
+		       , "transform (%d,%d,%d,%ld) ignored."
+		       , ike_info->ike_ealg
+		       , ike_info->ike_halg
+		       , ike_info->ike_modp
+		       , (long)ike_info->ike_eklen);
+	    } 
+
+	} else {
+	    struct db_sa *new;
+
+	    /* now merge emp_sa and gsp */
+	    if(gsp) {
+		new = sa_merge_proposals(gsp, emp_sp);
+		free_sa(gsp);
+		free_sa(emp_sp);
+		emp_sp = NULL;
+		gsp = new;
+	    } else {
+		gsp = emp_sp;
+	    }
+	}
+	transcnt++;
+    }
+fail:
+    return gsp;
+}
+
+
 /*
  * Local Variables:
  * c-style: pluto
