@@ -12,9 +12,12 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: state.h,v 1.98.2.3 2006-11-24 04:21:42 paul Exp $
+ * RCSID $Id: state.h,v 1.100 2005/08/05 19:16:49 mcr Exp $
  */
 
+#ifndef _STATE_H
+#define _STATE_H
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -43,9 +46,12 @@ typedef u_int32_t msgid_t;	/* Network order! */
 #define MAINMODE_MSGID    ((msgid_t) 0)
 
 struct state;	/* forward declaration of tag */
-extern bool reserve_msgid(struct state *isakmp_sa, msgid_t msgid);
+extern void reserve_msgid(struct state *isakmp_sa, msgid_t msgid);
+extern bool unique_msgid(struct state *isakmp_sa, msgid_t msgid);
+
 extern msgid_t generate_msgid(struct state *isakmp_sa);
 
+#define XAUTH_USERNAME_LEN 64
 
 /* Oakley (Phase 1 / Main Mode) transform and attributes
  * This is a flattened/decoded version of what is represented
@@ -74,6 +80,9 @@ struct oakley_trans_attrs {
  * This is a flattened/decoded version of what is represented
  * by a Transaction Payload.  There may be one for AH, one
  * for ESP, and a funny one for IPCOMP.
+ *
+ * Yes, this is screwy -- we keep different direction information
+ * in different places. Fix it up sometime.
  */
 struct ipsec_trans_attrs {
     u_int8_t transid;	/* transform id */
@@ -93,13 +102,43 @@ struct ipsec_trans_attrs {
 /* IPsec per protocol state information */
 struct ipsec_proto_info {
     bool present;	/* was this transform specified? */
-    struct ipsec_trans_attrs attrs;
+    struct ipsec_trans_attrs attrs;   /* info on remote */
     ipsec_spi_t our_spi;
     u_int16_t keymat_len;	/* same for both */
     u_char *our_keymat;
     u_char *peer_keymat;
 };
 
+/* internal state that
+ * should get copied by god
+ * Eistein would be proud
+ */
+
+struct hidden_variables {
+    unsigned int   st_malformed_received;
+    unsigned int   st_malformed_sent;
+    bool           st_xauth_client_done;
+    int            st_xauth_client_attempt;
+    bool           st_modecfg_server_done;
+    bool           st_modecfg_vars_set;
+    bool           st_got_certrequest;
+    bool           st_modecfg_started;
+    bool           st_skeyid_calculated;
+    bool           st_dpd;                 /* Peer supports DPD */
+    bool           st_dpd_local;	   /* If we want DPD on this conn */
+    bool           st_logged_p1algos;      /* if we have logged algos */
+    u_int32_t      st_nat_traversal;       /* bit field of permitted
+					    * methods. If non-zero, then
+					    * NAT-T has been detected, and
+					    * should be used. */
+    ip_address     st_nat_oa;
+    ip_address     st_natd;
+};                        
+
+#define set_suspended(st,md) do { st->st_suspended_md=md; \
+                                  st->st_suspended_md_func=__FUNCTION__; \
+                                  st->st_suspended_md_line=__LINE__; } while(0)
+
 /* state object: record the state of a (possibly nascent) SA
  *
  * Invariants (violated only during short transitions):
@@ -120,16 +159,20 @@ struct state
                                                 */
 
     struct msg_digest *st_suspended_md;        /* suspended state-transition */
+    const char        *st_suspended_md_func;
+    int                st_suspended_md_line;
 
     struct oakley_trans_attrs st_oakley;
 
     struct ipsec_proto_info st_ah;
     struct ipsec_proto_info st_esp;
     struct ipsec_proto_info st_ipcomp;
-#ifdef KLIPS
+
     ipsec_spi_t        st_tunnel_in_spi;          /* KLUDGE */
     ipsec_spi_t        st_tunnel_out_spi;         /* KLUDGE */
-#endif
+
+    IPsecSAref_t       ref;	   /* our kernel name for our incoming SA */
+    IPsecSAref_t       refhim;     /* our kernel name for our outgoing SA */
 
     const struct oakley_group_desc *st_pfs_group; /*group for Phase 2 PFS */
 
@@ -146,6 +189,7 @@ struct state
     u_int16_t          st_localport;           
 
     msgid_t            st_msgid;               /* MSG-ID from header.  Network Order! */
+    bool               st_reserve_msgid;       /* if TRUE, then message id has been reserved already */
 
     msgid_t            st_msgid_phase15;       /* msgid for phase 1.5 */
     msgid_t            st_msgid_phase15b;      /* msgid for phase 1.5 */
@@ -204,15 +248,20 @@ struct state
     u_int8_t           st_retransmit;          /* Number of retransmits */
     unsigned long      st_try;                 /* number of times rekeying attempted */
                                                /* 0 means the only time */
-    time_t             st_margin;              /* life after EVENT_SA_REPLACE */
+    time_t             st_margin;              /* life after EVENT_SA_REPLACE*/
     unsigned long      st_outbound_count;      /* traffic through eroute */
-    time_t             st_outbound_time;       /* time of last change to st_outbound_count */
+    time_t             st_outbound_time;       /* time of last change to
+						* st_outbound_count */
 
-    bool               st_calculating;         /* set to TRUE, if we are performing cryptographic
-						* operations on this state at this time
+    bool               st_calculating;         /* set to TRUE, if we are
+						* performing cryptographic
+						* operations on this state at
+						* this time
 						*/
 
-    chunk_t            st_p1isa;               /* Phase 1 initiator SA (Payload) for HASH */
+    chunk_t            st_p1isa;               /* Phase 1 initiator SA
+						  (Payload) for HASH
+					       */
     chunk_t            st_skeyid;              /* Key material */
     chunk_t            st_skeyid_d;            /* KM for non-ISAKMP key derivation */
     chunk_t            st_skeyid_a;            /* KM for ISAKMP authentication */
@@ -232,32 +281,10 @@ struct state
     struct state      *st_hashchain_next;      /* Next in list */
     struct state      *st_hashchain_prev;      /* Previous in list */
 
-    struct {
-        unsigned int   st_malformed_received;
-        unsigned int   st_malformed_sent;
-	bool           st_xauth_client_done;
-	int            st_xauth_client_attempt;
-        bool           st_modecfg_server_done;
-        bool           st_modecfg_vars_set;
-	bool           st_got_certrequest;
-        bool           st_modecfg_started;
-	bool           st_skeyid_calculated;
-	bool           st_dpd;                 /* Peer supports DPD */
-	bool           st_dpd_local;	       /* If we want DPD on this conn */
-	bool           st_logged_p1algos;      /* if we have logged algos */
-	u_int32_t      st_nat_traversal;       /* bit field of permitted
-						* methods. If non-zero, then
-						* NAT-T has been detected, and
-						* should be used. */
-	ip_address     st_nat_oa;
-	ip_address     st_natd;
-    } hidden_variables;                        /* internal state that
-						* should get copied by god
-						* Eistein would be proud
-						*/
+    struct hidden_variables hidden_variables;
 
-
-    unsigned char *st_xauth_username;
+    char                st_xauth_username[XAUTH_USERNAME_LEN];
+    chunk_t             st_xauth_password;
 
     /* RFC 3706 Dead Peer Detection */
     time_t              st_last_dpd;            /* Time of last DPD transmit */
@@ -333,6 +360,9 @@ extern void set_state_ike_endpoints(struct state *st
 				    , struct connection *c);
 
 extern void delete_cryptographic_continuation(struct state *st);
+extern void delete_states_dead_interfaces(void);
+
+#endif /* _STATE_H */
 
 /*
  * Local Variables:
