@@ -15,7 +15,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: x509keys.c,v 1.5 2005-06-14 22:38:06 mcr Exp $
+ * RCSID $Id: x509keys.c,v 1.6 2005/08/05 19:18:47 mcr Exp $
  */
 
 #include <stdlib.h>
@@ -29,8 +29,7 @@
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
 
-#include <sys/queue.h>
-
+#include "sysdep.h"
 #include "constants.h"
 #include "oswlog.h"
 
@@ -55,7 +54,6 @@
 #include "ocsp.h"
 #include "pkcs.h"
 #include "x509more.h"
-#include "paths.h"
 
 /* extract id and public key from x.509 certificate and
  * insert it into a pubkeyrec
@@ -79,8 +77,8 @@ add_x509_public_key(x509cert_t *cert , time_t until
     pk->dns_auth_level = dns_auth_level;
     pk->until_time = until;
     pk->issuer = cert->issuer;
-    delete_public_keys(&pk->id, pk->alg);
-    install_public_key(pk, &pubkeys);
+    delete_public_keys(&pluto_pubkeys, &pk->id, pk->alg);
+    install_public_key(pk, &pluto_pubkeys);
 
     gn = cert->subjectAltName;
 
@@ -96,8 +94,8 @@ add_x509_public_key(x509cert_t *cert , time_t until
 	    pk->dns_auth_level = dns_auth_level;
 	    pk->until_time = until;
 	    pk->issuer = cert->issuer;
-	    delete_public_keys(&pk->id, pk->alg);
-	    install_public_key(pk, &pubkeys);
+	    delete_public_keys(&pluto_pubkeys, &pk->id, pk->alg);
+	    install_public_key(pk, &pluto_pubkeys);
 	}
 	gn = gn->next;
     }
@@ -115,8 +113,8 @@ remove_x509_public_key(/*const*/ x509cert_t *cert)
     struct pubkey *revoked_pk;
 
     revoked_pk = allocate_RSA_public_key(c);
-    p          = pubkeys;
-    pp         = &pubkeys;
+    p          = pluto_pubkeys;
+    pp         = &pluto_pubkeys;
 
     while(p != NULL)
    {
@@ -153,32 +151,32 @@ decode_cert(struct msg_digest *md)
 	blob.len = pbs_left(&p->pbs);
 	if (cert->isacert_type == CERT_X509_SIGNATURE)
 	{
-	    x509cert_t cert = empty_x509cert;
-	    if (parse_x509cert(blob, 0, &cert))
+	    x509cert_t cert2 = empty_x509cert;
+	    if (parse_x509cert(blob, 0, &cert2))
 	    {
-		if (verify_x509cert(&cert, strict_crl_policy, &valid_until))
+		if (verify_x509cert(&cert2, strict_crl_policy, &valid_until))
 		{
 		    DBG(DBG_X509 | DBG_PARSING,
 			DBG_log("Public key validated")
 		    )
-		    add_x509_public_key(&cert, valid_until, DAL_SIGNED);
+		    add_x509_public_key(&cert2, valid_until, DAL_SIGNED);
 		}
 		else
 		{
 		    plog("X.509 certificate rejected");
 		}
-		free_generalNames(cert.subjectAltName, FALSE);
-		free_generalNames(cert.crlDistributionPoints, FALSE);
+		free_generalNames(cert2.subjectAltName, FALSE);
+		free_generalNames(cert2.crlDistributionPoints, FALSE);
 	    }
 	    else
 		plog("Syntax error in X.509 certificate");
 	}
 	else if (cert->isacert_type == CERT_PKCS7_WRAPPED_X509)
 	{
-	    x509cert_t *cert = NULL;
+	    x509cert_t *cert2 = NULL;
 
-	    if (parse_pkcs7_cert(blob, &cert))
-		store_x509certs(&cert, strict_crl_policy);
+	    if (parse_pkcs7_cert(blob, &cert2))
+		store_x509certs(&cert2, strict_crl_policy);
 	    else
 		plog("Syntax error in PKCS#7 wrapped X.509 certificates");
 	}
@@ -265,7 +263,7 @@ bool
 collect_rw_ca_candidates(struct msg_digest *md, generalName_t **top)
 {
     struct connection *d = find_host_connection(&md->iface->ip_addr
-	, pluto_port, (ip_address*)NULL, md->sender_port);
+	, pluto_port, (ip_address*)NULL, md->sender_port, LEMPTY);
 
     for (; d != NULL; d = d->hp_next)
     {
