@@ -1,8 +1,8 @@
-TESTNAME=fail-x509-07
+TESTNAME=fail-x509-08
 source /testing/pluto/bin/eastlocal.sh
 
 rm /tmp/$TESTNAME/ipsec.d/certs/west*
-rm /tmp/$TESTNAME/ipsec.d/crls/nic.crl
+rm /tmp/$TESTNAME/ipsec.d/crls/ca.crl
 
 iptables -A INPUT -i eth1 -s 192.0.3.0/24 -d 0.0.0.0/0 -j DROP
 
